package ch.unige.aou.controller.access;

import java.util.ArrayList;
import java.util.List;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import ch.unige.solidify.controller.IndexResourceReadOnlyController;
import ch.unige.solidify.rest.ActionName;
import ch.unige.solidify.rest.BooleanClauseType;
import ch.unige.solidify.rest.FacetPage;
import ch.unige.solidify.rest.FacetRequest;
import ch.unige.solidify.rest.RestCollection;
import ch.unige.solidify.rest.SearchCondition;
import ch.unige.solidify.rest.SearchConditionType;
import ch.unige.solidify.security.EveryonePermissions;
import ch.unige.solidify.service.IndexResourceService;

import ch.unige.aou.config.AouProperties;
import ch.unige.aou.controller.AccessController;
import ch.unige.aou.model.index.PublicationIndexEntry;
import ch.unige.aou.model.rest.AouSearchCondition;
import ch.unige.aou.rest.AouActionName;
import ch.unige.aou.rest.UrlPath;
import ch.unige.aou.service.QueryBuilderService;

@RestController
@EveryonePermissions
@ConditionalOnBean(AccessController.class)
@RequestMapping(UrlPath.ACCESS_METADATA)
public class SearchController extends IndexResourceReadOnlyController<String, PublicationIndexEntry> {

  private final String indexName;
  private final QueryBuilderService queryBuilderService;

  protected SearchController(AouProperties aouProperties, IndexResourceService<String, PublicationIndexEntry> indexResourceService,
          QueryBuilderService queryBuilderService) {
    super(indexResourceService);

    this.queryBuilderService = queryBuilderService;
    this.indexName = aouProperties.getIndexing().getPublishedPublicationsIndexName();
  }

  @Override
  public HttpEntity<RestCollection<PublicationIndexEntry>> list(@ModelAttribute PublicationIndexEntry search, Pageable pageable) {
    return super.list(search, pageable);
  }

  @Override
  public HttpEntity<PublicationIndexEntry> get(@PathVariable String id) {
    return super.get(id);
  }

  @PostMapping(value = "/" + ActionName.SEARCH)
  public HttpEntity<RestCollection<PublicationIndexEntry>> searchPost(@RequestBody List<AouSearchCondition> searchConditions,
          Pageable pageable) {

    List<FacetRequest> facetRequests = this.queryBuilderService.getFacetsList();
    List<SearchCondition> searchConditionList = this.queryBuilderService.replaceAliasesByIndexField(searchConditions);

    FacetPage<PublicationIndexEntry> results = this.indexResourceService.search(this.getIndex(), searchConditionList, facetRequests, pageable);
    return this.page2Collection(results, pageable);
  }

  @GetMapping(value = "/" + AouActionName.RUN_STORED_SEARCH + "/{storedSearchId}")
  public HttpEntity<RestCollection<PublicationIndexEntry>> runStoredSearch(@PathVariable String storedSearchId, Pageable pageable) {

    List<AouSearchCondition> searchConditions = this.queryBuilderService.getStoredSearchConditions(storedSearchId);

    return this.searchPost(searchConditions, pageable);
  }

  @GetMapping(value = "/" + AouActionName.COMBINE_STORED_SEARCHES)
  public HttpEntity<RestCollection<PublicationIndexEntry>> combineStoredResearches(@RequestParam("searches") String commaSeparatedResearchIds,
          @RequestParam(name = "combinationType", required = false, defaultValue = "or") String combinationType, Pageable pageable) {

    BooleanClauseType booleanClauseType = BooleanClauseType.SHOULD;
    if (combinationType.equalsIgnoreCase("and")) {
      booleanClauseType = BooleanClauseType.FILTER;
    }

    String[] researchIds = commaSeparatedResearchIds.split(",");

    List<AouSearchCondition> searchConditions = new ArrayList<>();
    for (String storedSearchId : researchIds) {
      AouSearchCondition aouSearchCondition = new AouSearchCondition();
      aouSearchCondition.setBooleanClauseType(booleanClauseType);
      aouSearchCondition.setType(SearchConditionType.NESTED_BOOLEAN);
      aouSearchCondition.getNestedConditions().addAll(this.queryBuilderService.getStoredSearchConditions(storedSearchId));
      searchConditions.add(aouSearchCondition);
    }

    return this.searchPost(searchConditions, pageable);
  }

  @Override
  protected String getIndex() {
    return this.indexName;
  }
}
