package ch.unige.aou.service;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.stereotype.Service;

import ch.unige.solidify.exception.SolidifyRuntimeException;
import ch.unige.solidify.model.index.IndexFieldAlias;
import ch.unige.solidify.rest.FacetRequest;
import ch.unige.solidify.rest.SearchCondition;
import ch.unige.solidify.rest.SearchConditionType;

import ch.unige.aou.config.AouProperties;
import ch.unige.aou.controller.AccessController;
import ch.unige.aou.model.rest.AouSearchCondition;
import ch.unige.aou.model.search.StoredSearch;
import ch.unige.aou.model.search.StoredSearchCriteria;
import ch.unige.aou.service.rest.trusted.TrustedIndexPropertiesRemoteResourceService;
import ch.unige.aou.service.rest.trusted.TrustedStoredSearchesRemoteResourceService;

@Service
@ConditionalOnBean(AccessController.class)
public class QueryBuilderService {

  private final String publishedPublicationsIndexName;

  private final TrustedIndexPropertiesRemoteResourceService trustedIndexFieldAliasRemoteResourceService;
  private final TrustedStoredSearchesRemoteResourceService trustedStoredSearchesRemoteResourceService;

  public QueryBuilderService(TrustedIndexPropertiesRemoteResourceService trustedIndexFieldAliasRemoteResourceService,
          TrustedStoredSearchesRemoteResourceService trustedStoredSearchesRemoteResourceService, AouProperties aouProperties) {
    this.trustedIndexFieldAliasRemoteResourceService = trustedIndexFieldAliasRemoteResourceService;
    this.trustedStoredSearchesRemoteResourceService = trustedStoredSearchesRemoteResourceService;
    this.publishedPublicationsIndexName = aouProperties.getIndexing().getPublishedPublicationsIndexName();
  }

  public List<FacetRequest> getFacetsList() {
    return this.trustedIndexFieldAliasRemoteResourceService.getFacetRequests(this.publishedPublicationsIndexName);
  }

  public List<AouSearchCondition> getStoredSearchConditions(String storedResearchId) {
    StoredSearch storedSearch = this.trustedStoredSearchesRemoteResourceService.findOneWithCache(storedResearchId);
    List<StoredSearchCriteria> criteriaList = storedSearch.getCriteria();
    List<AouSearchCondition> aouSearchConditions = new ArrayList<>();
    for (StoredSearchCriteria criteria : criteriaList) {
      aouSearchConditions.add(criteria.toAouSearchCondition());
    }
    return aouSearchConditions;
  }

  public List<SearchCondition> replaceAliasesByIndexField(List<AouSearchCondition> searchConditions) {
    List<IndexFieldAlias> indexFieldAliases = this.trustedIndexFieldAliasRemoteResourceService.getIndexFieldAliases(
            this.publishedPublicationsIndexName);

    for (AouSearchCondition searchCondition : searchConditions) {
      if (searchCondition.getType() == SearchConditionType.NESTED_BOOLEAN) {
        List<AouSearchCondition> nestedSearchConditions = searchCondition.getNestedConditions().stream().map(AouSearchCondition.class::cast)
                .collect(Collectors.toList());
        this.replaceAliasesByIndexField(nestedSearchConditions);
      } else {
        Optional<IndexFieldAlias> indexFieldAliasOpt = indexFieldAliases.stream().filter(f -> f.getAlias().equals(searchCondition.getField()))
                .findFirst();
        if (indexFieldAliasOpt.isPresent()) {
          searchCondition.setField(indexFieldAliasOpt.get().getField());
        } else {
          throw new SolidifyRuntimeException("Index field with alias '" + searchCondition.getField() + "' not found");
        }
      }
    }
    return searchConditions.stream().map(SearchCondition.class::cast).collect(Collectors.toList());
  }
}
