package ch.unige.aou.service;

import java.util.ArrayList;
import java.util.List;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import ch.unige.solidify.rest.RestCollectionPage;

import ch.unige.aou.config.AouProperties;
import ch.unige.aou.controller.AccessController;
import ch.unige.aou.model.publication.PublicationSubtypeDTO;
import ch.unige.aou.model.search.AdvancedSearchCriteria;
import ch.unige.aou.model.settings.LabelDTO;
import ch.unige.aou.service.rest.trusted.TrustedPublicationSubtypeRemoteResourceService;

@Service
@ConditionalOnBean(AccessController.class)
public class SettingsService {

  private final TrustedPublicationSubtypeRemoteResourceService trustedPublicationSubtypeRemoteResourceService;

  private final List<AdvancedSearchCriteria> advancedSearchCriteriaProperties;
  private List<AdvancedSearchCriteria> completedAdvancedSearchCriteriaList = new ArrayList<>();

  public SettingsService(AouProperties aouProperties,
          TrustedPublicationSubtypeRemoteResourceService trustedPublicationSubtypeRemoteResourceService) {
    this.trustedPublicationSubtypeRemoteResourceService = trustedPublicationSubtypeRemoteResourceService;
    this.advancedSearchCriteriaProperties = aouProperties.getIndexing().getAdvancedSearchCriteria();
  }

  public List<AdvancedSearchCriteria> getAdvancedSearchCriteriaList() {

    if (this.completedAdvancedSearchCriteriaList.isEmpty()) {
      this.completedAdvancedSearchCriteriaList = this.buildAdvancedSearchCriteria();
    }

    return this.completedAdvancedSearchCriteriaList;
  }

  private List<AdvancedSearchCriteria> buildAdvancedSearchCriteria() {
    List<AdvancedSearchCriteria> advancedSearchCriteriaList = new ArrayList<>();

    for (AdvancedSearchCriteria advancedSearchCriteria : this.advancedSearchCriteriaProperties) {

      if (advancedSearchCriteria.getBuildType() == AdvancedSearchCriteria.BuildType.DOCUMENT_TYPE) {
        this.fillDocumentTypes(advancedSearchCriteria);
      }

      advancedSearchCriteriaList.add(advancedSearchCriteria);
    }

    return advancedSearchCriteriaList;
  }

  private void fillDocumentTypes(AdvancedSearchCriteria advancedSearchCriteria) {
    List<PublicationSubtypeDTO> publicationSubtypes = this.trustedPublicationSubtypeRemoteResourceService.findAllDTO(
            PageRequest.of(0, RestCollectionPage.MAX_SIZE_PAGE, Sort.by("sortValue"))).getData();

    for (PublicationSubtypeDTO publicationSubtype : publicationSubtypes) {
      AdvancedSearchCriteria.EnumValue enumValue = new AdvancedSearchCriteria.EnumValue();
      enumValue.setValue(publicationSubtype.getName());
      for (LabelDTO label : publicationSubtype.getLabels()) {
        AdvancedSearchCriteria.Label asLabel = new AdvancedSearchCriteria.Label();
        asLabel.setLanguageCode(label.getLanguageCode());
        asLabel.setText(label.getText());
        enumValue.getLabels().add(asLabel);
      }
      advancedSearchCriteria.getSelectableValues().add(enumValue);
    }
  }
}
