package ch.unige.aou.business;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.stereotype.Service;

import ch.unige.aou.controller.AdminController;
import ch.unige.aou.model.notification.EventType;
import ch.unige.aou.repository.EventTypeRepository;
import ch.unige.aou.specification.EventTypeSpecification;

@Service
@ConditionalOnBean(AdminController.class)
public class EventTypeService extends AouResourceService<EventType> {
  private static final Logger log = LoggerFactory.getLogger(EventTypeService.class);

  private EventTypeRepository eventTypeRepository;

  public EventTypeService(EventTypeRepository eventTypeRepository) {
    this.eventTypeRepository = eventTypeRepository;
  }

  @Override
  public EventTypeSpecification getSpecification(EventType eventType) {
    return new EventTypeSpecification(eventType);
  }

  public void initDefaultData() {
    for (EventType eventType : EventType.getAllEventTypes()) {
      if (!this.eventTypeRepository.findById(eventType.getResId()).isPresent()) {
        this.eventTypeRepository.save(eventType);
        log.info("EventType {} created with name {}", eventType.getResId(), eventType.getName());
      }
    }
  }
}
