package ch.unige.aou.business;

import java.util.Arrays;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.stereotype.Service;

import ch.unige.aou.controller.AdminController;
import ch.unige.aou.model.notification.NotificationType;
import ch.unige.aou.repository.NotificationTypeRepository;
import ch.unige.aou.specification.NotificationTypeSpecification;

@Service
@ConditionalOnBean(AdminController.class)
public class NotificationTypeService extends AouResourceService<NotificationType> {
  private static final Logger log = LoggerFactory.getLogger(NotificationTypeService.class);

  private NotificationTypeRepository notificationTypeRepository;

  public NotificationTypeService(NotificationTypeRepository notificationTypeRepository) {
    this.notificationTypeRepository = notificationTypeRepository;
  }

  @Override
  public NotificationTypeSpecification getSpecification(NotificationType resource) {
    return new NotificationTypeSpecification(resource);
  }

  public void initDefaultData() {
    for (NotificationType notificationType : NotificationType.getAllNotificationTypes()) {
      if (!this.notificationTypeRepository.findById(notificationType.getResId()).isPresent()) {
        this.notificationTypeRepository.save(notificationType);
        log.info("NotificationType {} created with event {}", notificationType.getResId(), notificationType.getEventType().getResId());
      }
    }
  }

  public List<String> getAllMandatoryNotificationTypes() {
    return Arrays.asList(NotificationType.PUBLICATION_FORGOTTEN_IN_PROGRESS.getResId(), NotificationType.MY_PUBLICATION_FEEDBACK_REQUIRED.getResId(),
            NotificationType.PUBLICATION_FORGOTTEN_TO_VALIDATE.getResId());
  }

}
