package ch.unige.aou.business;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.stereotype.Service;

import ch.unige.solidify.service.JoinResource2TiersService;
import ch.unige.solidify.specification.Join2TiersSpecification;

import ch.unige.aou.controller.AdminController;
import ch.unige.aou.model.display.StructureWithJoinInfosDTO;
import ch.unige.aou.model.publication.Publication;
import ch.unige.aou.model.publication.PublicationStructure;
import ch.unige.aou.model.settings.Structure;
import ch.unige.aou.specification.PublicationStructureSpecification;

@Service
@ConditionalOnBean(AdminController.class)
public class PublicationStructureService extends JoinResource2TiersService<Publication, Structure, PublicationStructure> {

  @Override
  public Publication getEmptyParentResourceObject() {
    return new Publication();
  }

  @Override
  public Structure getEmptyChildResourceObject() {
    return new Structure();
  }

  @Override
  public Join2TiersSpecification<PublicationStructure> getJoinSpecification(PublicationStructure publicationStructure) {
    return new PublicationStructureSpecification(publicationStructure);
  }

  @Override
  public PublicationStructure getEmptyJoinResourceObject() {
    return new PublicationStructure();
  }

  @Override
  public void setParentResource(PublicationStructure publicationStructure, Publication publication) {
    publicationStructure.setPublication(publication);
  }

  @Override
  public void setChildResource(PublicationStructure publicationStructure, Structure structure) {
    publicationStructure.setStructure(structure);
  }

  @Override
  public Structure getChildResource(PublicationStructure publicationStructure) {
    return publicationStructure.getStructure();
  }

  @Override
  public PublicationStructure getDefaultJoinResource() {
    PublicationStructure publicationStructure = new PublicationStructure();

    // default linkType is VALIDATION as it is used in PublicationStructureController that allows to manage VALIDATION structures only
    publicationStructure.setLinkType(PublicationStructure.LinkType.VALIDATION);

    return publicationStructure;
  }

  @Override
  public String getChildPathFromJoinResource() {
    return PublicationStructure.PATH_TO_STRUCTURE;
  }

  @Override
  public StructureWithJoinInfosDTO getChildDTO(PublicationStructure joinResource) {
    return new StructureWithJoinInfosDTO(joinResource);
  }

  @Override
  public void deleteRelation(PublicationStructure joinResource) {
    joinResource.getStructure().getPublicationStructures().remove(joinResource);
    joinResource.getPublication().getPublicationStructures().remove(joinResource);
    super.deleteRelation(joinResource);
  }
}
