package ch.unige.aou.controller.admin;

import java.util.NoSuchElementException;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.controller.ResourceController;
import ch.unige.solidify.rest.ActionName;
import ch.unige.solidify.rest.RestCollection;
import ch.unige.solidify.security.EveryonePermissions;
import ch.unige.solidify.security.RootPermissions;

import ch.unige.aou.business.ContributorService;
import ch.unige.aou.controller.AdminController;
import ch.unige.aou.model.publication.Contributor;
import ch.unige.aou.model.search.ContributorSearchType;
import ch.unige.aou.rest.AouActionName;
import ch.unige.aou.rest.UrlPath;

@RootPermissions
@RestController
@ConditionalOnBean(AdminController.class)
@RequestMapping(UrlPath.ADMIN_CONTRIBUTORS)
public class ContributorsController extends ResourceController<Contributor> {

  @Override
  @GetMapping(SolidifyConstants.URL_SEP + ActionName.SEARCH)
  @ResponseBody
  @EveryonePermissions
  public HttpEntity<RestCollection<Contributor>> advancedSearch(@ModelAttribute Contributor contributor, @RequestParam("search") String search,
          @RequestParam(value = "match", required = false) String matchType, Pageable pageable) {
    return super.advancedSearch(contributor, search, matchType, pageable);
  }

  @GetMapping(value = "/" + ActionName.SEARCH, params = "term")
  @ResponseBody
  @EveryonePermissions
  public HttpEntity<RestCollection<Contributor>> searchContributor(@RequestParam("term") String term,
          @RequestParam(value = "type", required = false, defaultValue = "unige") String type, Pageable pageable) {
    ContributorSearchType searchType = ContributorSearchType.valueOf(type.toUpperCase());
    Page<Contributor> contributorPage = ((ContributorService) this.itemService).searchUnigeContributor(term, searchType, pageable);
    this.setResourceLinks(contributorPage);
    final RestCollection<Contributor> collection = this.setCollectionLinks(contributorPage, pageable);
    return new ResponseEntity<>(collection, HttpStatus.OK);
  }

  @GetMapping("/" + AouActionName.GET_NON_UNIGE_BY_NAME)
  @EveryonePermissions
  public HttpEntity<Contributor> getNonUnigeByName(@RequestParam("lastName") String lastName, @RequestParam("firstName") String firstName) {
    Contributor contributor = ((ContributorService) this.itemService).findByFirstNameAndLastNameAndCnIndividuIsNull(firstName, lastName);
    if (contributor == null) {
      throw new NoSuchElementException("No non unige contributor with name " + lastName + ", " + firstName);
    }
    this.addLinks(contributor);
    return new ResponseEntity<>(contributor, HttpStatus.OK);
  }

  @GetMapping("/" + AouActionName.GET_BY_CN_INDIVIDU)
  @EveryonePermissions
  public HttpEntity<Contributor> getByCnIndividu(@RequestParam("cnIndividu") String cnIndividu) {
    Contributor contributor = ((ContributorService) this.itemService).findByCnIndividu(cnIndividu);
    if (contributor == null) {
      throw new NoSuchElementException("No contributor with cnIndividu " + cnIndividu);
    }
    this.addLinks(contributor);
    return new ResponseEntity<>(contributor, HttpStatus.OK);
  }

}
