package ch.unige.aou.repository;

import java.time.OffsetDateTime;
import java.util.List;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

import ch.unige.solidify.model.GlobalBanner;
import ch.unige.solidify.repository.SolidifyRepository;

import ch.unige.aou.controller.AdminController;

@Repository
@SuppressWarnings("squid:S1214")
@ConditionalOnBean(AdminController.class)
public interface GlobalBannerRepository extends SolidifyRepository<GlobalBanner> {

  // *************
  // ** Queries **
  // *************

  GlobalBanner findByName(String name);

  @Query("SELECT gb "
          + "FROM GlobalBanner gb "
          + "WHERE gb.enabled is true "
          + "AND gb.startDate <= :dateTime "
          + "AND gb.endDate > :dateTime "
          + "ORDER BY gb.startDate ASC ")
  GlobalBanner findActive(OffsetDateTime dateTime);

  @Query("SELECT gb "
          + "FROM GlobalBanner gb "
          + "WHERE gb.enabled is true "
          + "AND ( "
          + "(:startDate >= gb.startDate AND :startDate < gb.endDate) "
          + "OR (:endDate > gb.startDate AND :endDate <= gb.endDate) "
          + "OR (:startDate <= gb.startDate AND :endDate >= gb.endDate) "
          + ") ")
  List<GlobalBanner> findActiveOverlaping(OffsetDateTime startDate, OffsetDateTime endDate);

}
