package ch.unige.aou.repository;

import java.util.List;
import java.util.Optional;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import ch.unige.solidify.repository.SolidifyRepository;

import ch.unige.aou.controller.AdminController;
import ch.unige.aou.model.settings.Person;
import ch.unige.aou.model.settings.Structure;

@Repository
@ConditionalOnBean(AdminController.class)
public interface StructureRepository extends SolidifyRepository<Structure> {
  Structure findByCnStructC(String cnStructC);

  Structure findByCodeStruct(String cStruct);

  List<Structure> findByNameAndParentStructureResId(String name, String parentStructureId);

  @Query("SELECT s.parentStructure.resId FROM Structure s WHERE s.resId = :resId")
  String getParentStructureResId(String resId);

  @Query("SELECT u FROM Structure s JOIN s.people u WHERE s.resId = :parentId AND u.resId=:id")
  Person findPersonByParentId(@Param("parentId") String parentId, @Param("id") String id);

  @Query("SELECT u FROM Structure s JOIN s.people u WHERE s.resId = :parentId")
  Page<Person> findPersonByParentId(@Param("parentId") String parentId, Pageable pageable);

  @Query("SELECT s FROM User u JOIN u.person p JOIN p.structures s WHERE u.externalUid like :externalUid")
  Page<Structure> findByUserExternalUid(@Param("externalUid") String externalUid, Pageable pageable);

  @Query(value = "SELECT max(st.sortValue) FROM Structure st WHERE st.parentStructure.resId = :parentStructureId")
  Optional<Integer> findMaxSortValueByParentStructure(String parentStructureId);

  @Query(value = "SELECT max(st.sortValue) FROM Structure st WHERE st.parentStructure.resId IS NULL")
  Optional<Integer> getMaxSortValue();
}
