package ch.unige.aou.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.context.annotation.Profile;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.stereotype.Service;

import ch.unige.aou.config.AouProperties;
import ch.unige.aou.controller.AdminController;
import ch.unige.aou.message.EmailMessage;

@Service
@ConditionalOnBean(AdminController.class)
@Profile("email-service")
public class EmailJmsListener extends MessageProcessor<EmailMessage> {

  private static final Logger log = LoggerFactory.getLogger(EmailJmsListener.class);

  private final EmailProcessingService emailProcessingService;

  public EmailJmsListener(AouProperties aouProperties, EmailProcessingService emailProcessingService) {
    super(aouProperties);
    this.emailProcessingService = emailProcessingService;
  }

  @JmsListener(destination = "${aou.queue.emails}")
  @Override
  public void receiveMessage(EmailMessage emailMessage) {
    this.sendForParallelProcessing(emailMessage);
  }

  @Override
  public void processMessage(EmailMessage emailMessage) {
    log.info("Reading message {}", emailMessage);
    this.emailProcessingService.processMessage(emailMessage);
    log.info("Message {} processed", emailMessage);
  }
}
