package ch.unige.aou.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.context.annotation.Profile;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.stereotype.Service;

import ch.unige.aou.config.AouProperties;
import ch.unige.aou.controller.AdminController;
import ch.unige.aou.message.PublicationFromFedoraMessage;

@Profile("storage-fedora3")
@Service
@ConditionalOnBean(AdminController.class)
public class PublicationFedoraJmsListener extends MessageProcessor<PublicationFromFedoraMessage> {

  private static final Logger log = LoggerFactory.getLogger(PublicationFedoraJmsListener.class);

  private final PublicationFedoraProcessingService publicationFedoraProcessingService;

  public PublicationFedoraJmsListener(AouProperties aouProperties, PublicationFedoraProcessingService publicationFedoraProcessingService) {
    super(aouProperties);
    this.publicationFedoraProcessingService = publicationFedoraProcessingService;
  }

  @JmsListener(destination = "${aou.queue.publicationsFromFedora}")
  @Override
  public void receiveMessage(PublicationFromFedoraMessage publicationFromFedoraMessage) {
    this.sendForParallelProcessing(publicationFromFedoraMessage);
  }

  @Override
  public void processMessage(PublicationFromFedoraMessage publicationFromFedoraMessage) {
    log.info("Reading message {} from pid {}", publicationFromFedoraMessage, publicationFromFedoraMessage.getPid());
    this.publicationFedoraProcessingService.processMessage(publicationFromFedoraMessage);
    log.info("Message {} processed from pid {}", publicationFromFedoraMessage, publicationFromFedoraMessage.getPid());
  }
}
