package ch.unige.aou.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import ch.unige.aou.business.UserService;
import ch.unige.aou.controller.AdminController;

@Service
@ConditionalOnBean(AdminController.class)
public class UserSynchronizationService {
  private static final Logger log = LoggerFactory.getLogger(UserSynchronizationService.class);

  private final UserService userService;

  protected UserSynchronizationService(UserService userService) {
    this.userService = userService;
  }

  @Scheduled(cron = "0 59 23 * * *")
  @Transactional
  public void synchronizeUsers() {
    try {
      // Update login info & role
      this.userService.synchronizeUsersInfoFromAuthServer();
      // Synchronize Orcid
      this.userService.synchronizeUsersOrcidWithAuthServer();
      log.info("Users synchronized successfully");
    } catch (RuntimeException e) {
      log.error("Users not synchronized");
    }
  }
}
