package ch.unige.aou.service.files;

import java.io.IOException;
import java.net.URL;

import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.pdmodel.encryption.InvalidPasswordException;
import org.apache.pdfbox.text.PDFTextStripper;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;

import ch.unige.aou.controller.AdminController;
import ch.unige.aou.model.publication.DocumentFile;

/**
 * FileMetadataExtractor implementation that supports only PDF files
 */
@Service
@ConditionalOnBean(AdminController.class)
public class PdfBoxMetadataExtractor implements FileMetadataExtractor {

  @Override
  public boolean isValidFile(DocumentFile pdfDocumentFile) {
    try (PDDocument doc = PDDocument.load(new URL(pdfDocumentFile.getFinalData().toString()).openStream())) {
      return true;
    } catch (InvalidPasswordException invalidPasswordException) {
      return true;
    } catch (IOException e) {
      return false;
    }
  }

  @Override
  public boolean isPasswordProtected(DocumentFile pdfDocumentFile) throws IOException {
    try (PDDocument doc = PDDocument.load(new URL(pdfDocumentFile.getFinalData().toString()).openStream())) {
      return false;
    } catch (InvalidPasswordException invalidPasswordException) {
      return true;
    }
  }

  @Override
  public String getText(DocumentFile pdfDocumentFile) throws IOException {
    if (MediaType.APPLICATION_PDF_VALUE.equals(pdfDocumentFile.getMimetype())) {
      if (!this.isPasswordProtected(pdfDocumentFile)) {
        try (PDDocument doc = PDDocument.load(new URL(pdfDocumentFile.getFinalData().toString()).openStream())) {
          return new PDFTextStripper().getText(doc);
        }
      }
    }
    return null;
  }
}
