package ch.unige.aou.service.security;

import java.util.NoSuchElementException;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.stereotype.Service;

import ch.unige.aou.business.PersonService;
import ch.unige.aou.business.StoredSearchService;
import ch.unige.aou.controller.AdminController;
import ch.unige.aou.controller.AouControllerAction;
import ch.unige.aou.model.search.StoredSearch;

@Service("storedSearchPermissionService")
@ConditionalOnBean(AdminController.class)
public class StoredSearchPermissionService extends AbstractAdminPermissionService {

  private StoredSearchService storedSearchService;

  public StoredSearchPermissionService(PersonService personService, StoredSearchService storedSearchService) {
    super(personService);
    this.storedSearchService = storedSearchService;
  }

  public boolean isAllowed(String storedSearchId, String actionName) {
    if (this.canBypassPermissionCheck()) {
      return true;
    }

    StoredSearch existingResource = this.getExistingResource(storedSearchId);
    if (existingResource == null) {
      throw new NoSuchElementException(storedSearchId);
    }

    String personId = this.getPersonAuthenticatedId();
    AouControllerAction action = this.getControllerAction(actionName);
    return this.isAllowedToPerformActionOnResource(personId, existingResource, action);
  }

  public boolean isAllowed(String[] storedSearchIds, String actionName) {
    for (String id : storedSearchIds) {
      if (!this.isAllowed(id, actionName)) {
        return false;
      }
    }
    return true;
  }

  /******************************************************************************************************/

  private StoredSearch getExistingResource(String resId) {
    return this.storedSearchService.findOne(resId);
  }

  /******************************************************************************************************/

  private boolean isAllowedToPerformActionOnResource(String personId, StoredSearch resourceToCheck, AouControllerAction action) {

    switch (action) {
      case UPDATE:
      case GET:
      case DELETE:
        return this.isCreator(resourceToCheck, personId);
      default:
        return false;
    }
  }

  private boolean isCreator(StoredSearch search, String personId) {
    return search.getCreator().getResId().equals(personId);
  }
}
