package ch.unige.aou.test;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.Mockito.when;

import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.validation.BeanPropertyBindingResult;
import org.springframework.validation.BindingResult;

import ch.unige.aou.AouMetadataVersion;
import ch.unige.aou.model.publication.Publication;
import ch.unige.aou.model.rest.UnigePersonDTO;
import ch.unige.aou.model.xml.deposit.v2_4.DepositDoc;
import ch.unige.aou.service.metadata.MetadataExtractorV2;
import ch.unige.aou.service.metadata.MetadataValidatorV2;
import ch.unige.aou.service.rest.RockUnigePersonSearchService;

public class MetadataValidatorV2Test extends MetadataValidatorServiceTest {

  private MetadataValidatorV2 metadataValidator;
  private MetadataExtractorV2 metadataExtractor;
  @Mock
  private RockUnigePersonSearchService unigePersonSearchService;

  protected static final String CN_INDIVIDU = "12345";

  @Override
  @BeforeEach
  public void setUp() {
    super.setUp();
    this.metadataExtractor = new MetadataExtractorV2(this.aouProperties, this.publicationTypeService, this.publicationSubtypeService,
            this.publicationSubSubtypeService, this.structureService, this.researchGroupService, this.contributorService,
            this.labeledLanguageService, this.journalTitleRemoteService, this.personService, this.messageService);

    this.metadataValidator = new MetadataValidatorV2(this.aouProperties, this.metadataExtractor, this.messageService,
            this.publicationTypeService, this.publicationSubtypeService, this.publicationSubSubtypeService, this.researchGroupService,
            this.structureService, this.duplicateService, this.unigePersonSearchService, this.documentFileService, this.historyService);
  }

  @Test
  void validateRulesForValidatorsWithoutErrorsTest() {
    /**
     * Create DepositDoc from Json form
     */
    String jsonFormData = this.getJsonFormData(AouMetadataVersion.getDefaultVersion());

    DepositDoc depositDoc = this.metadataExtractor.createDepositDocFromJsonFormData(jsonFormData);

    Publication publication = new Publication();
    final BindingResult errors = new BeanPropertyBindingResult(publication, publication.getClass().getName());

    UnigePersonDTO unigePersonDTO = new UnigePersonDTO(CN_INDIVIDU, "John", "Doe", 1980, "author", "structure");
    when(this.unigePersonSearchService.searchByCnIndividu(CN_INDIVIDU)).thenReturn(Optional.of(unigePersonDTO));

    this.metadataValidator.validateRulesForValidators(publication, depositDoc, errors);
    assertEquals(errors.getAllErrors().size(), 0); // there no errors
  }

  @Test
  void validateRulesForValidatorsWithErrorsTest() {
    /**
     * Create DepositDoc from Json form
     */
    String jsonFormData = this.getJsonFormData(AouMetadataVersion.getDefaultVersion());

    DepositDoc depositDoc = this.metadataExtractor.createDepositDocFromJsonFormData(jsonFormData);

    Publication publication = new Publication();
    final BindingResult errors = new BeanPropertyBindingResult(publication, publication.getClass().getName());
    Pageable pageable = PageRequest.of(0, 10);

    UnigePersonDTO unigePersonDTO = new UnigePersonDTO(CN_INDIVIDU, "Unkonw", "Unkonw", 1980, "author", "structure");
    when(this.unigePersonSearchService.searchByCnIndividu(CN_INDIVIDU)).thenReturn(Optional.of(unigePersonDTO));

    this.metadataValidator.validateRulesForValidators(publication, depositDoc, errors);
    assertEquals(errors.getAllErrors().size(), 2); // there are errors
  }
}
