package ch.unige.aou.service.admin;

import java.nio.file.Path;
import java.util.List;
import java.util.Map;

import org.springframework.core.env.Environment;
import org.springframework.core.io.Resource;
import org.springframework.stereotype.Service;

import ch.unige.solidify.SolidifyClientApplication;
import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.service.ResourceClientService;
import ch.unige.solidify.util.AbstractRestClientTool;

import ch.unige.aou.model.settings.Institution;
import ch.unige.aou.model.settings.Person;
import ch.unige.aou.rest.AouActionName;
import ch.unige.aou.rest.ModuleName;
import ch.unige.aou.rest.ResourceName;

@Service
public class InstitutionClientService extends ResourceClientService<Institution> {

  private static final String INSTITUTION_PATH = SolidifyConstants.URL_SEP + ResourceName.INSTITUTIONS;

  public InstitutionClientService(AbstractRestClientTool restClientTool, Environment env) {
    super(SolidifyClientApplication.getApplicationPrefix(), ModuleName.ADMIN, INSTITUTION_PATH, Institution.class, restClientTool, env);
  }

  public void addPerson(String institutionId, String personId) {
    this.addItem(institutionId, ResourceName.PEOPLE, personId, Person.class);
  }

  public List<Person> getPersons(String institutionId) {
    return this.findAllLinkedResources(institutionId, ResourceName.PEOPLE, Person.class);
  }

  public void removePerson(String institutionId, String personId) {
    this.removeItem(institutionId, ResourceName.PEOPLE, personId);
  }

  public Institution uploadLogo(String resId, Resource file) {
    return this.uploadImage(resId, AouActionName.UPLOAD_LOGO, file);
  }

  public void downloadLogo(String resId, Path path) {
    this.downloadImage(resId, AouActionName.DOWNLOAD_LOGO, path);
  }

  public void deleteLogo(String resId) {
    this.deleteAction(resId, AouActionName.DELETE_LOGO);
  }

  @Override
  public List<Institution> findAll() {
    return this.findAll(Institution.class);
  }

  @Override
  public List<Institution> searchByProperties(Map<String, String> properties) {
    return this.searchByProperties(properties, Institution.class);
  }

}
