package ch.unige.aou.service.admin;

import java.util.List;
import java.util.Map;

import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import ch.unige.solidify.SolidifyClientApplication;
import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.service.ResourceClientService;
import ch.unige.solidify.util.AbstractRestClientTool;
import ch.unige.solidify.util.CollectionTool;

import ch.unige.aou.model.notification.Notification;
import ch.unige.aou.rest.AouActionName;
import ch.unige.aou.rest.ModuleName;
import ch.unige.aou.rest.ResourceName;

@Service
public class NotificationClientService extends ResourceClientService<Notification> {

  public NotificationClientService(AbstractRestClientTool restClientTool, Environment env) {
    super(SolidifyClientApplication.getApplicationPrefix(), ModuleName.ADMIN, SolidifyConstants.URL_SEP + ResourceName.NOTIFICATIONS,
            Notification.class, restClientTool, env);
  }

  public List<Notification> getInboxNotifications() {
    return this.doActionOnList(AouActionName.INBOX, Notification.class);
  }

  public Notification getInboxNotification(String notificationId) {
    return this.doActionWithId(notificationId, AouActionName.INBOX);
  }

  public List<Notification> getInboxNotificationsByStructure(String structureId) {
    final String url = this.getResourceUrl() + SolidifyConstants.URL_SEP + AouActionName.SEARCH_INBOX + "?structureId=" + structureId;
    final RestTemplate restTemplate = this.restClientTool.getClient();
    final String jsonString = restTemplate.getForObject(url, String.class);
    return CollectionTool.getList(jsonString, Notification.class);
  }

  public List<Notification> getInboxNotificationsByResearchGroup(String researchGroupId) {
    final String url = this.getResourceUrl() + SolidifyConstants.URL_SEP + AouActionName.SEARCH_INBOX + "?researchGroupId=" + researchGroupId;
    final RestTemplate restTemplate = this.restClientTool.getClient();
    final String jsonString = restTemplate.getForObject(url, String.class);
    return CollectionTool.getList(jsonString, Notification.class);
  }

  public List<Notification> getSentNotifications() {
    return this.doActionOnList(AouActionName.SENT, Notification.class);
  }

  public Notification markAsRead(String notificationId) {
    final String url = this.getResourceUrl() + SolidifyConstants.URL_SEP + notificationId + SolidifyConstants.URL_SEP + AouActionName.SET_READ;
    final RestTemplate restTemplate = this.restClientTool.getClient();
    return restTemplate.postForObject(url, null, Notification.class);
  }

  @Override
  public List<Notification> findAll() {
    return this.findAll(Notification.class);
  }

  @Override
  public List<Notification> searchByProperties(Map<String, String> properties) {
    return this.searchByProperties(properties, Notification.class);
  }

}
