package ch.unige.aou.service.admin;

import java.nio.file.Path;
import java.util.List;

import org.springframework.core.io.Resource;

import com.fasterxml.jackson.databind.JsonNode;

import ch.unige.solidify.service.ResourceClientServiceInterface;

import ch.unige.aou.model.display.StructureValidationRight;
import ch.unige.aou.model.notification.NotificationType;
import ch.unige.aou.model.settings.Institution;
import ch.unige.aou.model.settings.Person;
import ch.unige.aou.model.settings.ResearchGroup;
import ch.unige.aou.model.settings.Structure;

public interface PersonClientServiceInterface extends ResourceClientServiceInterface<Person> {

  /**
   * Add a new institution to the OrganizationalUnit
   */
  void addInstitution(String personId, String institutionId);

  /**
   * Get list of institutions
   */
  List<Institution> getInstitutions(String personId);

  List<Person> search(String search, String matchType);

  List<Person> searchWithUser(String search);

  Person uploadAvatar(String resId, Resource resource);

  void downloadAvatar(String resId, Path path);

  void deleteAvatar(String resId);

  void addResearchGroup(String personId, String researchGroupId);

  List<ResearchGroup> getResearchGroup(String personId);

  void removeResearchGroup(String personId, String researchGroupId);

  List<Structure> getStructures(String personId);

  void addStructure(String personId, String structureId);

  void removeStructure(String personId, String structureId);

  void addValidationRightStructure(String personId, String structureId);

  void addValidationRightStructure(String personId, String... structureIds);

  void addValidationRightStructureWithSubtypes(String personId, String structureId, String... publicationSubtypeIds);

  void removeValidationRightStructure(String personId, String structureId);

  void removeValidationRightStructureWithSubtypes(String personId, String structureId, String... publicationSubtypeIds);

  List<StructureValidationRight> getValidationRightStructuresList(String personId);

  StructureValidationRight getValidationRightsStructure(String personId, String structureId);

  List<NotificationType> getSubscribedNotificationTypes(String personId);

  /*
   * Add notification type with default frequency: DAILY
   */
  void addNotificationType(String personId, String notificationTypeId);

  void removeNotificationType(String personId, String notificationTypeId);

  void addRole(String personId, String roleId);

  void removeRole(String personId, String roleId);

  /**
   * Returning as JsonNode is usefull to be able to check the "roleIds" property that is returned from
   * backend but that is readonly
   *
   * @param resId
   * @return
   */
  JsonNode findOneAsJsonNode(String resId);
}
