package ch.unige.aou.service.admin;

import java.nio.file.Path;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.core.env.Environment;
import org.springframework.core.io.Resource;
import org.springframework.stereotype.Service;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import ch.unige.solidify.SolidifyClientApplication;
import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.rest.ActionName;
import ch.unige.solidify.rest.Result;
import ch.unige.solidify.service.CompositionResourceClientService;
import ch.unige.solidify.util.AbstractRestClientTool;

import ch.unige.aou.model.publication.DocumentFile;
import ch.unige.aou.model.publication.Publication;
import ch.unige.aou.rest.AouActionName;
import ch.unige.aou.rest.ModuleName;
import ch.unige.aou.rest.ResourceName;

@Service
public class PublicationDocumentFileClientService extends CompositionResourceClientService<Publication, DocumentFile> {

  private static final String PUBLICATION_DOCUMENT_FILES_PATH = SolidifyConstants.URL_SEP + ResourceName.PUBLICATIONS
          + SolidifyConstants.URL_PARENT_ID + ResourceName.DOCUMENT_FILES;

  public PublicationDocumentFileClientService(AbstractRestClientTool restClientTool, Environment env) {
    super(SolidifyClientApplication.getApplicationPrefix(), ModuleName.ADMIN, PUBLICATION_DOCUMENT_FILES_PATH, DocumentFile.class,
            restClientTool, env);
  }

  public DocumentFile upload(String parentId, Resource file, MultiValueMap<String, Object> parameters) {
    String url = this.getResourceUrl();
    url = url.replace(SolidifyConstants.URL_PARENT_ID, SolidifyConstants.URL_SEP + parentId +
            SolidifyConstants.URL_SEP);
    url += SolidifyConstants.URL_SEP + ActionName.UPLOAD;

    return this.uploadFile(url, file, DocumentFile.class, parameters);
  }

  public void confirm(String parentId, String id) {
    String url = this.getResourceUrl() + SolidifyConstants.URL_SEP + id + SolidifyConstants.URL_SEP + AouActionName.CONFIRM;

    final Map<String, String> params = new HashMap<>();
    params.put(SolidifyConstants.URL_PARENT_ID_FIELD, parentId);
    final RestTemplate restTemplate = this.restClientTool.getClient();
    restTemplate.postForObject(url, null, Result.class, params);
  }

  public void downloadDocumentFile(String parentId, String id, Path path) {
    String url = this.getResourceUrl();
    url = url.replace(SolidifyConstants.URL_PARENT_ID, SolidifyConstants.URL_SEP + parentId + SolidifyConstants.URL_SEP);
    url += SolidifyConstants.URL_SEP + id + SolidifyConstants.URL_SEP + ActionName.DOWNLOAD;

    this.download(url, path);
  }

  @Override
  public List<DocumentFile> findAll(String parentId) {
    return this.findAll(parentId, DocumentFile.class);
  }

}
