package ch.unige.aou.service.admin;

import java.util.List;
import java.util.Map;

import org.springframework.core.env.Environment;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import ch.unige.solidify.SolidifyClientApplication;
import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.service.ResourceClientService;
import ch.unige.solidify.util.AbstractRestClientTool;

import ch.unige.aou.model.settings.Person;
import ch.unige.aou.model.settings.ResearchGroup;
import ch.unige.aou.rest.AouActionName;
import ch.unige.aou.rest.ModuleName;
import ch.unige.aou.rest.ResourceName;

@Service
public class ResearchGroupClientService extends ResourceClientService<ResearchGroup> {

  public ResearchGroupClientService(AbstractRestClientTool restClientTool, Environment env) {
    super(SolidifyClientApplication.getApplicationPrefix(), ModuleName.ADMIN, SolidifyConstants.URL_SEP + ResourceName.RESEARCH_GROUPS,
            ResearchGroup.class, restClientTool, env);
  }

  public void addPerson(String researchGroupId, String personId) {
    this.addItem(researchGroupId, ResourceName.PEOPLE, personId, Person.class);
  }

  public List<Person> getPeople(String researchGroupId) {
    return this.findAllLinkedResources(researchGroupId, ResourceName.PEOPLE, Person.class);
  }

  public void removePerson(String researchGroupId, String personId) {
    this.removeItem(researchGroupId, ResourceName.PEOPLE, personId);
  }

  public ResearchGroup validate(String resId) {
    String url = this.getResourceUrl() + SolidifyConstants.URL_SEP + resId + SolidifyConstants.URL_SEP + AouActionName.VALIDATE;
    final RestTemplate restTemplate = this.restClientTool.getClient();
    return restTemplate.exchange(url, HttpMethod.POST, null, ResearchGroup.class).getBody();
  }

  @Override
  public List<ResearchGroup> findAll() {
    return super.findAll(ResearchGroup.class);
  }

  @Override
  public List<ResearchGroup> searchByProperties(Map<String, String> properties) {
    return super.searchByProperties(properties, ResearchGroup.class);
  }

}
