package ch.unige.aou.filter;

import java.io.IOException;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import ch.unige.aou.config.AouProperties;

/**
 * Spring OAUTH2 and CORS Configuration
 * https://medium.com/@muiruri/spring-oauth2-and-cors-configuration-3529337525b4
 */
@Component
@Order(Ordered.HIGHEST_PRECEDENCE)
public class SimpleCorsFilter implements Filter {

  private String allowedDomains;
  private String allowedHeaders;
  private String allowedMethods;

  public SimpleCorsFilter(AouProperties aouProperties) {
    this.allowedDomains = aouProperties.getSecurity().getCors().getAllowedDomains();
    this.allowedHeaders = aouProperties.getSecurity().getCors().getAllowedHeaders();
    this.allowedMethods = aouProperties.getSecurity().getCors().getAllowedMethods();
  }

  @Override
  public void destroy() {
    // Nothing to do
  }

  @Override
  public void doFilter(ServletRequest req, ServletResponse res, FilterChain chain) throws IOException, ServletException {
    final HttpServletResponse response = (HttpServletResponse) res;
    final HttpServletRequest request = (HttpServletRequest) req;
    final String origin = request.getHeader("Origin");
    for (final String allowedOrigin : this.allowedDomains.split(" ")) {
      if (allowedOrigin.equals(origin)) {
        response.setHeader("Access-Control-Allow-Origin", allowedOrigin);
      }
    }
    response.setHeader("Access-Control-Allow-Methods", this.allowedMethods);
    response.setHeader("Access-Control-Max-Age", "3600");
    response.setHeader("Access-Control-Allow-Headers", this.allowedHeaders);

    if ("OPTIONS".equalsIgnoreCase(request.getMethod())) {
      response.setStatus(HttpServletResponse.SC_OK);
    } else {
      chain.doFilter(req, res);
    }
  }

  @Override
  public void init(FilterConfig filterConfig) {
    // Nothing to do
  }
}
