package ch.unige.aou.controller;

import java.util.List;

import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import ch.unige.solidify.business.IndexFieldAliasService;
import ch.unige.solidify.controller.ModuleController;
import ch.unige.solidify.index.settings.IndexingSettingsService;
import ch.unige.solidify.model.index.IndexFieldAlias;
import ch.unige.solidify.model.index.IndexSettings;

import ch.unige.aou.config.AouProperties;
import ch.unige.aou.rest.ModuleName;
import ch.unige.aou.rest.UrlPath;

@RestController
@ConditionalOnProperty(prefix = "aou.module.data-mgmt", name = "enable")
@RequestMapping(UrlPath.DATA_MGMT)
public class DataMgmtController extends ModuleController {

  private final IndexFieldAliasService indexFieldAliasService;

  protected DataMgmtController(AouProperties aouProperties, IndexingSettingsService<IndexSettings> indexSettingsService,
          IndexFieldAliasService indexFieldAliasService) {
    super(ModuleName.DATA_MGMT);

    this.indexFieldAliasService = indexFieldAliasService;

    // Module initialization
    if (aouProperties.getData().isInitIndexes()) {
      // Index creation
      indexSettingsService.init(aouProperties.getIndexing().getIndexDefinitionList());
    }

    if (aouProperties.getData().isInit()) {
      this.initIndexFieldAliases(aouProperties.getIndexing().getFieldAliases());
    }
  }

  private void initIndexFieldAliases(List<IndexFieldAlias> indexFieldAliases) {
    int facetPosition = 10;
    for (IndexFieldAlias indexFieldAlias : indexFieldAliases) {
      boolean isSystem = indexFieldAlias.isSystem() != null ? indexFieldAlias.isSystem() : false;
      boolean isFacet = indexFieldAlias.isFacet() != null ? indexFieldAlias.isFacet() : true;
      this.indexFieldAliasService.createIfNotExists(indexFieldAlias.getIndexName(), facetPosition, indexFieldAlias.getAlias(),
              indexFieldAlias.getField(), isFacet, isSystem, indexFieldAlias.getFacetMinCount(), indexFieldAlias.getFacetLimit());
      facetPosition += 10;
    }
  }
}
