package ch.unige.aou.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.stereotype.Service;

import ch.unige.aou.config.AouProperties;
import ch.unige.aou.controller.DataMgmtController;
import ch.unige.aou.message.PublicationIndexingMessage;

@Service
@ConditionalOnBean(DataMgmtController.class)
public class IndexingJmsListener extends MessageProcessor<PublicationIndexingMessage> {

  private static final Logger log = LoggerFactory.getLogger(IndexingJmsListener.class);

  private final IndexingProcessingService indexingProcessingService;

  public IndexingJmsListener(AouProperties aouProperties, IndexingProcessingService indexingProcessingService) {
    super(aouProperties);
    this.indexingProcessingService = indexingProcessingService;
  }

  @JmsListener(destination = "${aou.queue.publicationsIndexing}")
  @Override
  public void receiveMessage(PublicationIndexingMessage publicationIndexingMessage) {
    this.sendForParallelProcessing(publicationIndexingMessage);
  }

  @Override
  public void processMessage(PublicationIndexingMessage publicationIndexingMessage) {
    log.info("Reading message {}", publicationIndexingMessage);
    this.indexingProcessingService.indexPublication(publicationIndexingMessage.getResId());
    log.debug("Publication '{}' indexed", publicationIndexingMessage.getResId());
  }
}
