package ch.unige.aou.test.admin;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.util.List;

import org.junit.jupiter.api.Test;
import org.springframework.web.client.HttpClientErrorException;

import ch.unige.aou.model.settings.LicenseGroup;

public class LicenseGroupAsAdminIT extends LicenseIT {
  @Override
  protected void setUser() {
    this.restClientTool.sudoAdmin();
  }

  @Test
  void findAllTest() {
    LicenseGroup licenseGroup = this.createTestLicenseGroup("Test group");

    // Find all
    final List<LicenseGroup> actualLicenseGroups = this.licenseGroupService.findAll();

    // Test at least
    assertFalse(actualLicenseGroups.isEmpty());
    assertTrue(actualLicenseGroups.stream().anyMatch(group -> group.getResId().equals(licenseGroup.getResId())));
  }

  @Test
  void creationTest() {
    LicenseGroup licenseGroup = this.createTestLicenseGroup("Test group");

    final LicenseGroup foundLicenseGroup = this.licenseGroupService.findOne(licenseGroup.getResId());
    assertNotNull(foundLicenseGroup);
    this.assertsLicenseGroup(licenseGroup, foundLicenseGroup);
  }

  @Test
  void deleteTest() {
    LicenseGroup licenseGroup = this.createTestLicenseGroup("Test group");

    LicenseGroup foundLicenseGroup = this.licenseGroupService.findOne(licenseGroup.getResId());
    assertNotNull(foundLicenseGroup);

    this.licenseGroupService.delete(licenseGroup.getResId());

    final String resId = licenseGroup.getResId();
    assertThrows(HttpClientErrorException.NotFound.class, () -> this.licenseGroupService.findOne(resId));
  }

  @Test
  void updateTest() {
    LicenseGroup licenseGroup = this.createTestLicenseGroup("Test group");

    LicenseGroup foundLicenseGroup = this.licenseGroupService.findOne(licenseGroup.getResId());
    assertNotNull(foundLicenseGroup);

    String newName = this.getTemporaryTestLabel("updated");
    foundLicenseGroup.setName(newName);

    LicenseGroup updatedLicenseGroup = this.licenseGroupService.update(foundLicenseGroup.getResId(), foundLicenseGroup);

    LicenseGroup refoundLicenseGroup = this.licenseGroupService.findOne(licenseGroup.getResId());
    assertNotNull(refoundLicenseGroup);

    assertEquals(newName, refoundLicenseGroup.getName());
  }

}
