package ch.unige.aou.test.admin;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.fail;

import java.util.List;

import org.junit.jupiter.api.Test;
import org.springframework.http.HttpStatus;
import org.springframework.web.client.HttpClientErrorException;

import ch.unige.aou.model.notification.Event;
import ch.unige.aou.model.security.Role;
import ch.unige.aou.test.AouTestConstants;

class RoleAsAdminIT extends AbstractAdminIT {

  @Override
  protected void setUser() {
    this.restClientTool.sudoAdmin();
  }

  @Test
  void creationTest() {
    final Role role = new Role();
    role.setName(AouTestConstants.getRandomNameWithTemporaryLabel("Role"));

    HttpClientErrorException e = assertThrows(HttpClientErrorException.class, () -> {
      this.roleService.create(role);
    });
    assertEquals(HttpStatus.FORBIDDEN, e.getStatusCode());
  }

  private Role createRoleAsRoot(String name) {
    Role role = new Role();
    role.setName(AouTestConstants.getRandomNameWithTemporaryLabel(name));

    this.restClientTool.sudoRoot();
    Role roleCreated = this.roleService.create(role);
    this.restClientTool.exitSudo();

    assertNotNull(roleCreated);

    return roleCreated;
  }

  private void deleteRoleAsRoot(Role role) {
    this.restClientTool.sudoRoot();
    this.roleService.delete(role.getResId());
    this.restClientTool.exitSudo();
  }

  @Test
  void deleteTest() {
    Role role = this.createRoleAsRoot("Role for admin delete test");

    // Delete
    final String resId = role.getResId();
    HttpClientErrorException e = assertThrows(HttpClientErrorException.class, () -> {
      this.roleService.delete(resId);
    });
    assertEquals(HttpStatus.FORBIDDEN, e.getStatusCode());

    this.deleteRoleAsRoot(role);
  }

  @Test
  void findAllTest() {
    List<Role> actualRoles = this.roleService.findAll();
    assertEquals(0, actualRoles.size());

    Role role = this.createRoleAsRoot("Role for admin find all test");

    // Find all
    actualRoles = this.roleService.findAll();

    // Test all roles
    assertEquals(1, actualRoles.size());

    this.deleteRoleAsRoot(role);
  }

  @Test
  void updateTest() {
    Role role = this.createRoleAsRoot("Role for admin update test");

    // Update the role name
    final String newRoleName = AouTestConstants.getRandomNameWithTemporaryLabel("New Role name set by admin");
    role.setName(newRoleName);

    String localRoleId = role.getResId();
    HttpClientErrorException e = assertThrows(HttpClientErrorException.class, () -> {
      this.roleService.update(localRoleId, role);
    });
    assertEquals(HttpStatus.FORBIDDEN, e.getStatusCode());

    this.deleteRoleAsRoot(role);
  }
}
