package ch.unige.aou.test.metadataImports;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.junit.jupiter.api.Assertions.fail;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;

import org.junit.jupiter.api.Test;
import org.springframework.core.io.ClassPathResource;

import ch.unige.solidify.util.FileTool;

import ch.unige.aou.model.publication.DocumentFile;
import ch.unige.aou.model.publication.Publication;
import ch.unige.aou.test.AbstractIT;

public class ArxivImportIT extends AbstractIT {
  private static final String JSON_FORM_DATA_FILE = "angular-form-data-with-arxiv.json";

  @Override
  protected void createFixtures() {
    this.restClientTool.sudoRoot();
    this.createPublicationTypeAndSubtypeFixtures(false);
    this.restClientTool.sudoRoot();
  }

  @Test()
  public void testImportFile() {
    //create a publication
    String jsonFormData = this.getJsonFormData();
    Publication publication = this.createPublicationWithSubType(this.getTemporaryTestLabel("Publication to import arxiv file"));
    publication.setImportSource(Publication.ImportSource.ARXIV);
    publication.setFormData(jsonFormData);
    this.publicationService.create(publication);

    //check document files for this publication
    List<DocumentFile> documentFileList = this.publicationDocumentFileService.findAll(publication.getResId());
    DocumentFile foundDocumentFile = this
            .waitForDocumentFileProcessingToBeOver(documentFileList.get(0).getPublication().getResId(), documentFileList.get(0).getResId());

    assertTrue(documentFileList.size() > 0);

    assertEquals(DocumentFile.AccessLevel.PUBLIC, documentFileList.get(0).getAccessLevel());
  }

  private String getJsonFormData() {
    try (InputStream jsonStream = new ClassPathResource(JSON_FORM_DATA_FILE).getInputStream()) {
      return FileTool.toString(jsonStream);
    } catch (IOException e) {
      fail("Unable to read " + JSON_FORM_DATA_FILE + ": " + e.getMessage());
      return null;
    }
  }

  @Override
  protected void deleteFixtures() {
    this.restClientTool.sudoRoot();
    this.clearPublicationFixtures();
    this.clearPublicationSubtypeFixtures();
    this.clearPublicationTypeFixtures();
    this.restClientTool.exitSudo();
  }
}
