package ch.unige.aou.model.notification;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import javax.persistence.Entity;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.validation.constraints.NotNull;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.unige.aou.model.AouResourceNormalized;
import ch.unige.aou.model.settings.Person;

@Entity
public class NotificationType extends AouResourceNormalized {

  // For User
  public static final NotificationType MY_PUBLICATION_COMMENTED = new NotificationType("MY_PUBLICATION_COMMENTED",
          EventType.COMMENT_IN_PUBLICATION);
  public static final NotificationType MY_INDIRECT_PUBLICATION_COMMENTED = new NotificationType("MY_INDIRECT_PUBLICATION_COMMENTED",
          EventType.COMMENT_IN_PUBLICATION);
  public static final NotificationType MY_PUBLICATION_REJECTED = new NotificationType("MY_PUBLICATION_REJECTED", EventType.PUBLICATION_REJECTED);
  public static final NotificationType MY_PUBLICATION_VALIDATED = new NotificationType("MY_PUBLICATION_VALIDATED",
          EventType.PUBLICATION_SUBMITTED);
  public static final NotificationType MY_PUBLICATION_FEEDBACK_REQUIRED = new NotificationType("MY_PUBLICATION_FEEDBACK_REQUIRED",
          EventType.PUBLICATION_FEEDBACK_REQUIRED);
  public static final NotificationType MY_INDIRECT_PUBLICATION_VALIDATED = new NotificationType("MY_INDIRECT_PUBLICATION_VALIDATED",
          EventType.PUBLICATION_SUBMITTED);
  public static final NotificationType PUBLICATION_FORGOTTEN_IN_PROGRESS = new NotificationType("PUBLICATION_FORGOTTEN_IN_PROGRESS",
          EventType.PUBLICATION_CREATED);

  // For Validator
  public static final NotificationType PUBLICATION_TO_VALIDATE = new NotificationType("PUBLICATION_TO_VALIDATE",
          EventType.PUBLICATION_TO_VALIDATE);
  public static final NotificationType PUBLICATION_TO_VALIDATE_COMMENTED = new NotificationType("PUBLICATION_TO_VALIDATE_COMMENTED",
          EventType.COMMENT_IN_PUBLICATION);
  public static final NotificationType PUBLICATION_FORGOTTEN_TO_VALIDATE = new NotificationType("PUBLICATION_FORGOTTEN_TO_VALIDATE",
          EventType.PUBLICATION_TO_VALIDATE);

  @NotNull
  @ManyToOne
  private EventType eventType;

  @ManyToMany(mappedBy = "subscribedNotificationTypes")
  @JsonIgnore
  private Set<Person> subscribers;

  public NotificationType(String resId, EventType eventType) {
    this.setResId(resId);
    this.eventType = eventType;
  }

  public NotificationType() {
  }

  public static List<NotificationType> getAllNotificationTypes() {
    return Arrays.asList(PUBLICATION_TO_VALIDATE, PUBLICATION_TO_VALIDATE_COMMENTED, MY_PUBLICATION_REJECTED,
            MY_PUBLICATION_VALIDATED, MY_INDIRECT_PUBLICATION_VALIDATED, MY_PUBLICATION_FEEDBACK_REQUIRED, PUBLICATION_FORGOTTEN_TO_VALIDATE,
            PUBLICATION_FORGOTTEN_IN_PROGRESS, MY_INDIRECT_PUBLICATION_COMMENTED, MY_PUBLICATION_COMMENTED);
  }

  public Set<Person> getSubscribers() {
    return this.subscribers;
  }

  public boolean addSubscriber(Person p) {
    final boolean result = this.subscribers.add(p);
    if (!p.getSubscribedNotifications().contains(this)) {
      p.addNotificationType(this);
    }
    return result;
  }

  public boolean removeSubscriber(Person p) {
    final boolean result = this.subscribers.remove(p);
    if (p.getSubscribedNotifications().contains(this)) {
      p.removeNotificationType(this);
    }
    return result;
  }

  @Override
  public boolean equals(final Object o) {
    if (this == o)
      return true;
    if (!(o instanceof NotificationType))
      return false;
    final NotificationType that = (NotificationType) o;
    return this.getResId().equals(that.getResId()) &&
            Objects.equals(this.eventType, that.eventType);
  }

  @Override
  public int hashCode() {
    return Objects.hash(super.hashCode(), this.eventType);
  }

  public EventType getEventType() {
    return this.eventType;
  }

  public void setEventType(EventType eventType) {
    this.eventType = eventType;
  }
}
