package ch.unige.aou.model.oai;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.PrePersist;
import javax.persistence.PreUpdate;
import javax.persistence.Transient;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.unige.solidify.message.CacheMessage;
import ch.unige.solidify.rest.ResourceNormalized;

import ch.unige.aou.AouConstants;
import ch.unige.aou.rest.ModuleName;

@Entity
public class OAISet extends ResourceNormalized {

  @Size(min = 1, max = AouConstants.DB_LONG_STRING_LENGTH)
  private String description;

  @NotNull
  private Boolean enabled;

  @NotNull
  @Size(min = 1)
  @Column(unique = true)
  private String name;

  /**
   * The query on the index service (e.g Elasticsearch) used to populate the OAI set
   */
  @NotNull
  @Size(min = 1, max = AouConstants.DB_LONG_STRING_LENGTH)
  private String query;

  @NotNull
  @Size(min = 1)
  @Column(unique = true)
  private String spec;

  /****************************************************************/

  public OAISet() {

  }

  @Override
  @Transient
  @JsonIgnore
  public CacheMessage getCacheMessage() {
    final CacheMessage cacheMessage = super.getCacheMessage();
    cacheMessage.addOtherProperty("search", Boolean.TRUE);
    return cacheMessage;
  }

  public String getDescription() {
    return this.description;
  }

  public String getName() {
    return this.name;
  }

  public String getQuery() {
    return this.query;
  }

  /****************************************************************/

  public String getSpec() {
    return this.spec;
  }

  /****************************************************************/

  @Transient
  @JsonIgnore
  public Boolean hasEnabledValue() {
    return (this.enabled != null);
  }

  @Override
  @PrePersist
  @PreUpdate
  public void init() {
    if (this.enabled == null) {
      this.enabled = false;
    }
  }

  public boolean isEnabled() {
    if (this.enabled != null) {
      return this.enabled;
    } else {
      return Boolean.FALSE;
    }
  }

  @Override
  public String managedBy() {
    return ModuleName.ACCESS;
  }

  public void setDescription(String description) {
    this.description = description;
  }

  public void setEnabled(Boolean enabled) {
    this.enabled = enabled;
  }

  public void setName(String name) {
    this.name = name;
  }

  public void setQuery(String query) {
    this.query = query;
  }

  public void setSpec(String spec) {
    this.spec = spec;
  }

}
