package ch.unige.aou.model.publication;

import java.util.HashSet;
import java.util.Set;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.ManyToMany;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import org.springframework.data.jpa.domain.Specification;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.unige.solidify.util.SearchCriteria;
import ch.unige.solidify.util.StringTool;

import ch.unige.aou.model.AouSearchableResourceNormalized;
import ch.unige.aou.specification.ContributorAdvancedSearchSpecification;

@Entity
public class Contributor extends AouSearchableResourceNormalized<Contributor> {

  private String firstName;

  @NotNull
  @Size(min = 1)
  private String lastName;

  @Size(min = 1)
  @Column(unique = true)
  private String cnIndividu;

  @JsonIgnore
  @ManyToMany(mappedBy = "contributors")
  private Set<Publication> publications = new HashSet<>();

  /****************************************************************/

  public String getFirstName() {
    return this.firstName;
  }

  public String getLastName() {
    return this.lastName;
  }

  public String getCnIndividu() {
    return this.cnIndividu;
  }

  public Set<Publication> getPublications() {
    return this.publications;
  }

  public String getFullName() {
    String fullName = this.lastName;
    if (!StringTool.isNullOrEmpty(this.firstName)) {
      fullName = fullName + ", " + this.firstName;
    }
    return fullName;
  }

  /****************************************************************/

  public void setLastName(String lastName) {
    this.lastName = lastName;
  }

  public void setFirstName(String firstName) {
    this.firstName = firstName;
  }

  public void setCnIndividu(String cnIndividu) {
    this.cnIndividu = cnIndividu;
  }

  public void setPublications(Set<Publication> publications) {
    this.publications = publications;
  }

  @Override
  public Specification<Contributor> getSearchSpecification(SearchCriteria criteria) {
    return new ContributorAdvancedSearchSpecification(criteria);
  }
}
