package ch.unige.aou.model.search;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonIgnore;

public class AdvancedSearchCriteria implements Serializable {

  private static final long serialVersionUID = 831316416135184911L;

  public enum BuildType {
    NONE,           // no treatment to do when building the advanced search criteria list
    DOCUMENT_TYPE   // get list of documents types as selectable values
  }

  public enum CriteriaType {
    FREE_TEXT_TERM,        // free text value, search as SearchConditionType.TERM
    FREE_TEXT_MATCH,       // free text value, search as SearchConditionType.MATCH
    ENUM_SINGLE,           // closed list of values, only one can be selected
    ENUM_MULTIPLE,         // closed list of values, many can be selected
    INTERVAL_YEAR,         // range of years
    STRUCTURE,             // academic structures
    RESEARCH_GROUP,        // research groups
    CONTRIBUTOR_UNIGE,     // UNIGE contributors
    CONTRIBUTOR_NON_UNIGE, // external contributors (without cnIndividu)
  }

  public static class EnumValue implements Serializable {

    private static final long serialVersionUID = 83177896135184911L;

    private String value;
    private List<Label> labels = new ArrayList<>();

    public String getValue() {
      return this.value;
    }

    public void setValue(String value) {
      this.value = value;
    }

    public List<Label> getLabels() {
      return this.labels;
    }

    public void setLabels(List<Label> labels) {
      this.labels = labels;
    }
  }

  public static class Label implements Serializable {

    private static final long serialVersionUID = 6589746416135184911L;

    private String text;
    private String languageCode;

    public String getText() {
      return this.text;
    }

    public void setText(String text) {
      this.text = text;
    }

    public String getLanguageCode() {
      return this.languageCode;
    }

    public void setLanguageCode(String languageCode) {
      this.languageCode = languageCode;
    }
  }

  private String alias;
  private List<Label> labels = new ArrayList<>();
  private CriteriaType type;
  @JsonIgnore
  private BuildType buildType = BuildType.NONE;
  private List<EnumValue> selectableValues = new ArrayList<>();

  public String getAlias() {
    return this.alias;
  }

  public void setAlias(String alias) {
    this.alias = alias;
  }

  public List<Label> getLabels() {
    return this.labels;
  }

  public void setLabels(List<Label> labels) {
    this.labels = labels;
  }

  public CriteriaType getType() {
    return this.type;
  }

  public void setType(CriteriaType type) {
    this.type = type;
  }

  public BuildType getBuildType() {
    return this.buildType;
  }

  public void setBuildType(BuildType buildType) {
    this.buildType = buildType;
  }

  public List<EnumValue> getSelectableValues() {
    return this.selectableValues;
  }

  public void setSelectableValues(List<EnumValue> selectableValues) {
    this.selectableValues = selectableValues;
  }
}
