package ch.unige.aou.model.search;

import java.util.ArrayList;
import java.util.List;
import javax.persistence.Convert;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.ManyToOne;
import javax.persistence.PrePersist;
import javax.persistence.PreUpdate;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.converter.KeywordsConverter;
import ch.unige.solidify.rest.BooleanClauseType;
import ch.unige.solidify.rest.SearchConditionType;

import ch.unige.aou.model.AouResourceNormalized;
import ch.unige.aou.model.rest.AouSearchCondition;

@Entity
public class StoredSearchCriteria extends AouResourceNormalized {

  @NotNull
  @ManyToOne(targetEntity = StoredSearch.class)
  @JsonIgnore
  private StoredSearch search;

  @NotNull
  @Enumerated(EnumType.STRING)
  private CriteriaSource source;

  @NotNull
  @Enumerated(EnumType.STRING)
  private SearchConditionType type = SearchConditionType.MATCH;

  @NotNull
  @Enumerated(EnumType.STRING)
  private BooleanClauseType booleanClauseType = BooleanClauseType.MUST;

  @NotNull
  @Size(min = 1, max = SolidifyConstants.DB_DEFAULT_STRING_LENGTH)
  private String field;

  @Size(min = 1, max = SolidifyConstants.DB_DEFAULT_STRING_LENGTH)
  private String value;

  @Convert(converter = KeywordsConverter.class)
  private final List<String> terms = new ArrayList<>();

  @Size(min = 1, max = SolidifyConstants.DB_SHORT_STRING_LENGTH)
  private String lowerValue;

  @Size(min = 1, max = SolidifyConstants.DB_SHORT_STRING_LENGTH)
  private String upperValue;

  /****************************************************************/

  @Override
  @PrePersist
  @PreUpdate
  public void init() {
    this.clean();
    if (this.source == null) {
      this.source = CriteriaSource.ADVANCED_SEARCH;
    }
  }

  public void clean() {
    switch (this.type) {
      case MATCH:
      case QUERY:
      case SIMPLE_QUERY:
        this.terms.clear();
        this.lowerValue = null;
        this.upperValue = null;
        break;
      case TERM:
        this.lowerValue = null;
        this.upperValue = null;
        break;
      case RANGE:
        this.value = null;
        this.terms.clear();
        break;
      case NESTED_BOOLEAN:
        this.value = null;
        this.terms.clear();
        this.lowerValue = null;
        this.upperValue = null;
        break;
    }
  }

  /****************************************************************/

  public StoredSearch getSearch() {
    return this.search;
  }

  public void setSearch(StoredSearch search) {
    this.search = search;
  }

  public CriteriaSource getSource() {
    return this.source;
  }

  public void setSource(CriteriaSource source) {
    this.source = source;
  }

  public SearchConditionType getType() {
    return this.type;
  }

  public void setType(SearchConditionType type) {
    this.type = type;
  }

  public BooleanClauseType getBooleanClauseType() {
    return this.booleanClauseType;
  }

  public void setBooleanClauseType(BooleanClauseType booleanClauseType) {
    this.booleanClauseType = booleanClauseType;
  }

  public String getField() {
    return this.field;
  }

  public void setField(String field) {
    this.field = field;
  }

  public String getValue() {
    return this.value;
  }

  public void setValue(String value) {
    this.value = value;
  }

  public List<String> getTerms() {
    return this.terms;
  }

  public String getLowerValue() {
    return this.lowerValue;
  }

  public void setLowerValue(String lowerValue) {
    this.lowerValue = lowerValue;
  }

  public String getUpperValue() {
    return this.upperValue;
  }

  public void setUpperValue(String upperValue) {
    this.upperValue = upperValue;
  }

  /****************************************************************/

  public AouSearchCondition toAouSearchCondition() {
    AouSearchCondition aouSearchCondition = new AouSearchCondition();
    aouSearchCondition.setSource(this.getSource());
    aouSearchCondition.setField(this.getField());
    aouSearchCondition.setBooleanClauseType(this.getBooleanClauseType());
    aouSearchCondition.setLowerValue(this.getLowerValue());
    aouSearchCondition.setUpperValue(this.getUpperValue());
    aouSearchCondition.setType(this.getType());
    aouSearchCondition.setValue(this.getValue());
    aouSearchCondition.getTerms().addAll(this.getTerms());
    return aouSearchCondition;
  }
}
