package ch.unige.aou.model.security;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import javax.persistence.CollectionTable;
import javax.persistence.Column;
import javax.persistence.ElementCollection;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToMany;
import javax.persistence.UniqueConstraint;
import javax.validation.constraints.NotNull;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.unige.aou.AouConstants;
import ch.unige.aou.model.AouResourceNormalized;
import ch.unige.aou.model.Sortable;
import ch.unige.aou.model.settings.Label;
import ch.unige.aou.model.settings.Person;

@Entity
public class Role extends AouResourceNormalized implements Sortable {

  @NotNull
  @Column(unique = true)
  private String name;

  /**
   * Property allowing to sort roles in lists
   */
  @Column(nullable = false)
  private Integer sortValue;

  @ElementCollection
  @Column(name = AouConstants.DB_ROLE_ID)
  @CollectionTable(joinColumns = { @JoinColumn(name = AouConstants.DB_ROLE_ID) },
                   uniqueConstraints = @UniqueConstraint(columnNames = { AouConstants.DB_ROLE_ID, AouConstants.DB_LANGUAGE_ID }))
  private List<Label> labels = new ArrayList<>();

  @ManyToMany(mappedBy = "roles")
  @JsonIgnore
  private Set<Person> people;

  /****************************************************************/

  public String getName() {
    return this.name;
  }

  @Override
  public Integer getSortValue() {
    return this.sortValue;
  }

  public List<Label> getLabels() {
    return this.labels;
  }

  /****************************************************************/

  public void setName(String name) {
    this.name = name;
  }

  @Override
  public void setSortValue(Integer sortValue) {
    this.sortValue = sortValue;
  }

  public void setLabels(List<Label> labels) {
    this.labels = labels;
  }
}
