package ch.unige.aou.model.settings;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Convert;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToMany;
import javax.persistence.OneToOne;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import org.springframework.hateoas.server.mvc.WebMvcLinkBuilder;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.unige.solidify.converter.WeakManyToManyConverter;
import ch.unige.solidify.rest.ResourceNormalized;

import ch.unige.aou.AouConstants;
import ch.unige.aou.model.Image;
import ch.unige.aou.model.ImageInterface;
import ch.unige.aou.rest.AouActionName;
import ch.unige.aou.rest.ModuleName;
import ch.unige.aou.rest.ResourceName;

@Entity
public class Institution extends ResourceNormalized implements ImageInterface {
  @Size(max = AouConstants.DB_LONG_STRING_LENGTH)
  private String description;

  @Convert(converter = WeakManyToManyConverter.class)
  private List<String> emailSuffixes = new ArrayList<>();

  @NotNull
  @Column(unique = true)
  @Size(min = 1)
  private String name;

  @ManyToMany(mappedBy = "institutions")
  @JsonIgnore
  private Set<Person> people;

  private URL url;

  @OneToOne(cascade = CascadeType.ALL, orphanRemoval = true)
  @JoinColumn(name = AouConstants.DB_LOGO_ID, referencedColumnName = AouConstants.DB_RES_ID, nullable = true)
  private InstitutionLogo logo;

  @Override
  public void addLinks(WebMvcLinkBuilder linkBuilder, boolean mainRes, boolean subResOnly) {
    super.addLinks(linkBuilder, mainRes, subResOnly);
    if (mainRes) {
      this.add(linkBuilder.slash(this.getResId()).slash(ResourceName.PEOPLE).withRel(ResourceName.PEOPLE));
      this.add(linkBuilder.slash(this.getResId()).slash(AouActionName.UPLOAD_LOGO).withRel(AouActionName.UPLOAD_LOGO));
      this.add(linkBuilder.slash(this.getResId()).slash(AouActionName.DOWNLOAD_LOGO).withRel(AouActionName.DOWNLOAD_LOGO));
    }
  }

  public boolean addPerson(Person p) {
    final boolean result = this.people.add(p);
    if (!p.getInstitutions().contains(this)) {
      p.addInstitution(this);
    }
    return result;
  }

  public String getDescription() {
    return this.description;
  }

  public List<String> getEmailSuffixes() {
    return this.emailSuffixes;
  }

  public String getName() {
    return this.name;
  }

  public Set<Person> getPeople() {
    return this.people;
  }

  public URL getUrl() {
    return this.url;
  }

  @Override
  public void init() {
    // Do nothing
  }

  @Override
  public String managedBy() {
    return ModuleName.ADMIN;
  }

  public boolean removePerson(Person p) {
    final boolean result = this.people.remove(p);
    if (p.getInstitutions().contains(this)) {
      p.removeInstitution(this);
    }
    return result;
  }

  public void setDescription(String description) {
    this.description = description;
  }

  public void setEmailSuffixes(List<String> emailSuffixes) {
    this.emailSuffixes = emailSuffixes;
  }

  public void setName(String name) {
    this.name = name;
  }

  public void setUrl(URL url) {
    this.url = url;
  }

  public InstitutionLogo getLogo() {
    return this.logo;
  }

  public void setLogo(InstitutionLogo logo) {
    this.logo = logo;
  }

  @Override
  @JsonIgnore
  public Image getNewIfNeededImage() {
    if (this.getLogo() == null) {
      this.setLogo(new InstitutionLogo());
    }
    return this.getLogo();
  }

  @Override
  @JsonIgnore
  public Image getImage() {
    return this.getLogo();
  }

  @Override
  public void setImage(Image logo) {
    this.setLogo((InstitutionLogo) logo);
  }
}
