package ch.unige.aou.model.xml.deposit.v1.deserializer;

import java.io.IOException;
import java.math.BigInteger;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.ObjectCodec;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonNode;

import ch.unige.solidify.util.StringTool;

import ch.unige.aou.model.xml.deposit.AouStdDeserializer;
import ch.unige.aou.model.xml.deposit.v1.Container;
import ch.unige.aou.model.xml.deposit.v1.ContainerIdentifiers;
import ch.unige.aou.model.xml.deposit.v1.Text;

public class ContainerDeserializer extends AouStdDeserializer<Container> {

  private static final long serialVersionUID = 659865L;

  @Override
  public Container deserialize(JsonParser parser, DeserializationContext context) throws IOException, JsonProcessingException {

    ObjectCodec codec = parser.getCodec();
    JsonNode jsonNode = codec.readTree(parser);

    Container container = new Container();

    boolean hasValue = false;

    JsonNode titleNode = jsonNode.get("title");
    if (titleNode != null) {
      Text titleText = codec.treeToValue(titleNode, Text.class);
      if (titleText != null && !StringTool.isNullOrEmpty(titleText.getContent())) {
        container.setTitle(titleText);
        hasValue = true;
      }
    }

    JsonNode editorNode = jsonNode.get("editor");
    if (this.nodeHasTextValue(editorNode)) {
      container.setEditor(editorNode.textValue());
      hasValue = true;
    }

    JsonNode placeNode = jsonNode.get("place");
    if (this.nodeHasTextValue(placeNode)) {
      container.setPlace(placeNode.textValue());
      hasValue = true;
    }

    JsonNode volumeNode = jsonNode.get("volume");
    if (volumeNode != null && !volumeNode.isNull() && !StringTool.isNullOrEmpty(volumeNode.asText())) {
      container.setVolume(new BigInteger(volumeNode.asText()));
      hasValue = true;
    }

    JsonNode issueNode = jsonNode.get("issue");
    if (issueNode != null && !issueNode.isNull() && !StringTool.isNullOrEmpty(issueNode.asText())) {
      container.setIssue(new BigInteger(issueNode.asText()));
      hasValue = true;
    }

    JsonNode conferenceDateNode = jsonNode.get("conferenceDate");
    if (this.nodeHasTextValue(conferenceDateNode)) {
      container.setConferenceDate(conferenceDateNode.textValue());
      hasValue = true;
    }

    JsonNode specialIssueNode = jsonNode.get("specialIssue");
    if (this.nodeHasTextValue(specialIssueNode)) {
      container.setSpecialIssue(specialIssueNode.textValue());
      hasValue = true;
    }

    JsonNode identifiersNode = jsonNode.get("identifiers");
    if (identifiersNode != null) {
      ContainerIdentifiers containerIdentifiers = codec.treeToValue(identifiersNode, ContainerIdentifiers.class);
      if (containerIdentifiers != null) {
        container.setIdentifiers(containerIdentifiers);
        hasValue = true;
      }
    }

    if (hasValue) {
      return container;
    } else {
      return null;
    }
  }
}
