package ch.unige.aou.model.xml.deposit.v1.deserializer;

import java.io.IOException;
import java.math.BigInteger;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.ObjectCodec;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonNode;

import ch.unige.solidify.exception.SolidifyValidationException;
import ch.unige.solidify.util.StringTool;
import ch.unige.solidify.validation.ValidationError;

import ch.unige.aou.model.tool.ValidationTool;
import ch.unige.aou.model.xml.deposit.AouStdDeserializer;
import ch.unige.aou.model.xml.deposit.v1.DepositIdentifiers;
import ch.unige.aou.model.xml.deposit.v1.Dois;

public class DepositIdentifiersDeserializer extends AouStdDeserializer<DepositIdentifiers> {

  private static final long serialVersionUID = 659865L;

  @Override
  public DepositIdentifiers deserialize(JsonParser parser, DeserializationContext context) throws IOException, JsonProcessingException {

    ObjectCodec codec = parser.getCodec();
    JsonNode jsonNode = codec.readTree(parser);

    DepositIdentifiers depositIdentifiers = new DepositIdentifiers();

    boolean hasValue = false;

    JsonNode doisNode = jsonNode.get("dois");
    if (doisNode != null) {
      Dois dois = codec.treeToValue(doisNode, Dois.class);
      if (dois != null && !dois.getDoi().isEmpty()) {
        depositIdentifiers.setDois(dois);
        hasValue = true;
      }
    }

    JsonNode pmidNode = jsonNode.get("pmid");
    if (pmidNode != null && !pmidNode.isNull() && !StringTool.isNullOrEmpty(pmidNode.asText())) {
      if (!ValidationTool.isValidPMID(pmidNode.asText())) {
        ValidationError error = new ValidationError("Invalid PMID");
        error.addError("description.identifiers.pmid", "Invalid PMID");
        throw new SolidifyValidationException(error);
      }
      depositIdentifiers.setPmid(new BigInteger(pmidNode.asText()));
      hasValue = true;
    }

    JsonNode isbnNode = jsonNode.get("isbn");
    if (this.nodeHasTextValue(isbnNode)) {
      depositIdentifiers.setIsbn(isbnNode.textValue());
      hasValue = true;
    }

    JsonNode arxivNode = jsonNode.get("arxiv");
    if (this.nodeHasTextValue(arxivNode)) {
      depositIdentifiers.setArxiv(arxivNode.textValue());
      hasValue = true;
    }

    JsonNode reroNode = jsonNode.get("rero");
    if (this.nodeHasTextValue(reroNode)) {
      depositIdentifiers.setRero(reroNode.textValue());
      hasValue = true;
    }

    JsonNode repecNode = jsonNode.get("repec");
    if (this.nodeHasTextValue(repecNode)) {
      depositIdentifiers.setRepec(repecNode.textValue());
      hasValue = true;
    }

    JsonNode dblpNode = jsonNode.get("dblp");
    if (this.nodeHasTextValue(dblpNode)) {
      depositIdentifiers.setDblp(dblpNode.textValue());
      hasValue = true;
    }

    JsonNode urnNode = jsonNode.get("urn");
    if (this.nodeHasTextValue(urnNode)) {
      depositIdentifiers.setUrn(urnNode.textValue());
      hasValue = true;
    }

    JsonNode localNumberNode = jsonNode.get("localNumber");
    if (this.nodeHasTextValue(localNumberNode)) {
      depositIdentifiers.setLocalNumber(localNumberNode.textValue());
      hasValue = true;
    }

    if (hasValue) {
      return depositIdentifiers;
    } else {
      return null;
    }
  }

}
