package ch.unige.aou.model.xml.deposit.v2_2.deserializer;

import java.io.IOException;
import java.util.Objects;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.ObjectCodec;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;

import ch.unige.solidify.exception.SolidifyRuntimeException;

import ch.unige.aou.model.xml.deposit.AouStdDeserializer;
import ch.unige.aou.model.xml.deposit.v2_2.Abstracts;
import ch.unige.aou.model.xml.deposit.v2_2.Text;

public class AbstractsDeserializer extends AouStdDeserializer<Abstracts> {

  private static final long serialVersionUID = 65924865L;

  @Override
  public Abstracts deserialize(JsonParser parser, DeserializationContext context) throws IOException, JsonProcessingException {

    ObjectCodec codec = parser.getCodec();
    JsonNode jsonNode = codec.readTree(parser);

    if (jsonNode instanceof ArrayNode) {
      if (jsonNode.isEmpty()) {
        return null;
      }

      Abstracts abstracts = new Abstracts();
      for (JsonNode abstractNode : jsonNode) {
        Text abstractObj = codec.treeToValue(abstractNode, Text.class);

        //if list contain one element and all properties of this only element are empty we ignore the list
        if (jsonNode.size() == 1 && abstractObj == null) {
          return null;
        }

        abstracts.getAbstract().add(Objects.requireNonNullElse(abstractObj, new Text()));
      }
      return abstracts;

    } else {
      throw new SolidifyRuntimeException("abstracts is not an array");
    }
  }
}
