package ch.unige.aou.model.xml.deposit.v2_2.deserializer;

import java.io.IOException;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.ObjectCodec;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;

import ch.unige.solidify.exception.SolidifyRuntimeException;

import ch.unige.aou.model.xml.deposit.AouStdDeserializer;
import ch.unige.aou.model.xml.deposit.v2_2.Classification;
import ch.unige.aou.model.xml.deposit.v2_2.Classifications;

public class ClassificationsDeserializer extends AouStdDeserializer<Classifications> {

  private static final long serialVersionUID = 659865L;

  @Override
  public Classifications deserialize(JsonParser parser, DeserializationContext context) throws IOException, JsonProcessingException {

    ObjectCodec codec = parser.getCodec();
    JsonNode jsonNode = codec.readTree(parser);

    if (jsonNode instanceof ArrayNode) {
      if (jsonNode.isEmpty()) {
        return null;
      }

      Classifications classifications = new Classifications();
      for (JsonNode collectionNode : jsonNode) {
        Classification classification = new Classification();

        JsonNode itemNode = collectionNode.get("item");
        JsonNode codeNode = collectionNode.get("code");

        //if list contain one element and all properties of this only element are empty we ignore the list
        if (jsonNode.size() == 1 && !this.nodeHasValue(itemNode) && !this.nodeHasValue(codeNode)) {
          return null;
        }

        if (this.nodeHasTextValue(itemNode)) {
          classification.setItem(itemNode.textValue());
        }

        if (this.nodeHasTextValue(codeNode)) {
          classification.setCode(codeNode.textValue());
        }

        classifications.getClassification().add(classification);
      }
      return classifications;

    } else {
      throw new SolidifyRuntimeException("classifications is not an array");
    }
  }
}
