package ch.unige.aou.config;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.task.AsyncTaskExecutor;
import org.springframework.scheduling.annotation.AsyncConfigurerSupport;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.web.servlet.config.annotation.AsyncSupportConfigurer;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

@Configuration
@EnableAsync
public class AouAsyncConfig extends AsyncConfigurerSupport {

  private long defaultAsyncExecutionTimeout;

  public AouAsyncConfig(AouProperties aouProperties) {
    this.defaultAsyncExecutionTimeout = aouProperties.getParameters().getDefaultAsyncExecutionTimeout();
  }

  /**
   * Configure async support for Spring MVC.
   */
  @Bean
  public WebMvcConfigurer webMvcConfigurerAdapter(AsyncTaskExecutor taskExecutor) {

    return new WebMvcConfigurer() {
      @Override
      public void configureAsyncSupport(AsyncSupportConfigurer configurer) {

        // set no time limit for async executions to allow downloading very large files with
        // StreamingResponseBody
        configurer.setDefaultTimeout(AouAsyncConfig.this.defaultAsyncExecutionTimeout).setTaskExecutor(taskExecutor);
      }
    };
  }
}
