package ch.unige.aou.controller;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLConnection;

import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.method.annotation.StreamingResponseBody;

import ch.unige.solidify.exception.SolidifyRuntimeException;
import ch.unige.solidify.rest.Resource;
import ch.unige.solidify.security.UserPermissions;
import ch.unige.solidify.util.StringTool;

import ch.unige.aou.model.ImageInterface;

@UserPermissions
public abstract class AouResourceWithLogoController<T extends Resource & ImageInterface> extends AouResourceController<T> {

  @ResponseBody
  public HttpEntity<StreamingResponseBody> downloadLogo(@PathVariable String id) throws IOException {
    final T item = this.itemService.findOne(id);
    if (item.getImage() == null) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }

    InputStream is = new BufferedInputStream(new ByteArrayInputStream(item.getImage().getImage()));

    String mimeType = item.getImage().getMimeType();
    if (StringTool.isNullOrEmpty(mimeType)) {
      mimeType = URLConnection.guessContentTypeFromStream(is);
    }

    return this.buildDownloadResponseEntity(is, item.getImage().getFileName(), mimeType, item.getImage().getFileSize());
  }

  public HttpEntity<T> uploadLogo(@PathVariable String id, @RequestParam("file") MultipartFile file) throws IOException {
    T item = this.itemService.findOne(id);
    try {
      // create or update the logo
      item = this.updateOrCreateLogo(item, file);
    } catch (final IOException e) {
      final String message = "Error uploading file " + file;
      throw new SolidifyRuntimeException(message, e);
    }

    this.itemService.save(item);
    return new ResponseEntity<>(item, HttpStatus.OK);
  }

  public HttpEntity<T> deleteLogo(@PathVariable String id) throws IOException {
    T item = this.itemService.findOne(id);
    item.setImage(null);

    this.itemService.save(item);
    return new ResponseEntity<>(item, HttpStatus.OK);
  }

  private T updateOrCreateLogo(T item, MultipartFile file) throws IOException {
    item.getNewIfNeededImage().setImage(file.getBytes());
    item.getNewIfNeededImage().setFileSize(file.getSize());
    item.getNewIfNeededImage().setFileName(file.getOriginalFilename());
    item.getNewIfNeededImage().setMimeType(file.getContentType());
    return item;
  }
}
