package ch.unige.aou.repository;

import java.time.OffsetDateTime;
import java.util.List;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.PagingAndSortingRepository;
import org.springframework.data.repository.query.Param;

import ch.unige.aou.AouConstants;
import ch.unige.aou.model.StatusHistory;


public interface HistoryRepository extends PagingAndSortingRepository<StatusHistory, Long> {

  // *************
  // ** Queries **
  // *************

  public List<StatusHistory> findByResId(@Param(AouConstants.DB_RES_ID) String resId);

  public Page<StatusHistory> findByResId(@Param(AouConstants.DB_RES_ID) String resId, Pageable pageable);

  @Query("SELECT s FROM StatusHistory s WHERE s.resId = :" + AouConstants.DB_RES_ID + " ORDER BY s.changeTime DESC")
  public List<StatusHistory> findStatusHistoryOrderByLastDate(@Param(AouConstants.DB_RES_ID) String resId);

  @Query("SELECT CASE WHEN (COUNT(s) > 0) THEN TRUE ELSE FALSE END FROM StatusHistory s WHERE s.resId=:resId AND s.status=:status")
  boolean hasBeenInStatus(String resId, String status);

  @Query("SELECT s FROM StatusHistory s WHERE s.resId=:resId AND s.status=:status AND s.changeTime<:before ORDER BY s.changeTime DESC")
  List<StatusHistory> findAllWithStatusBeforeOrderByChangeTimeDesc(String resId, String status, OffsetDateTime before);

  @Query("SELECT s FROM StatusHistory s WHERE s.resId=:resId AND s.status=:status AND s.changeTime>:after ORDER BY s.changeTime ASC")
  List<StatusHistory> findAllWithStatusAfterOrderByChangeTimeAsc(String resId, String status, OffsetDateTime after);

  // ***************
  // ** Relations **
  // ***************

}
