package ch.unige.aou.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.context.annotation.Lazy;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import ch.unige.solidify.message.CacheMessage;
import ch.unige.solidify.message.ResourceCacheMessage;
import ch.unige.solidify.model.index.IndexFieldAlias;

import ch.unige.aou.model.publication.DocumentFileType;
import ch.unige.aou.model.publication.PublicationSubtype;
import ch.unige.aou.model.settings.Structure;

@Service
public class CacheService {
  private static final Logger log = LoggerFactory.getLogger(CacheService.class);

  protected CacheService self;

  public CacheService(@Lazy CacheService cacheService) {
    this.self = cacheService;
  }

  @JmsListener(destination = "${aou.topic.cache}", containerFactory = "topicListenerFactory")
  public void processCacheMessage(CacheMessage message) {
    if (log.isDebugEnabled()) {
      log.debug(message.toString());
    }

    if (message instanceof ResourceCacheMessage) {
      if (((ResourceCacheMessage) message).getResourceClass() == IndexFieldAlias.class) {
        this.self.evictCachedFacets();
        this.self.evictCachedIndexFieldAliases();
      }
      if (((ResourceCacheMessage) message).getResourceClass() == Structure.class) {
        this.self.evictCachedStructures();
      }
      if (((ResourceCacheMessage) message).getResourceClass() == PublicationSubtype.class) {
        this.self.evictCachedPublicationSubtypes();
      }
      if (((ResourceCacheMessage) message).getResourceClass() == DocumentFileType.class) {
        this.self.evictCachedDocumentFileTypes();
      }
    }
  }

  @CacheEvict(value = AouCacheNames.FACET_REQUEST, allEntries = true)
  public void evictCachedFacets() {
    log.info("Facets removed from cache");
  }

  @CacheEvict(value = AouCacheNames.INDEX_FIELD_ALIAS, allEntries = true)
  public void evictCachedIndexFieldAliases() {
    log.info("IndexFieldAliases removed from cache");
  }

  @CacheEvict(value = AouCacheNames.STRUCTURES, allEntries = true)
  public void evictCachedStructures() {
    log.info("Structures removed from cache");
  }

  @CacheEvict(value = AouCacheNames.PUBLICATION_SUBTYPES, allEntries = true)
  public void evictCachedPublicationSubtypes() {
    log.info("PublicationSubtypes removed from cache");
  }

  @CacheEvict(value = AouCacheNames.DOCUMENT_FILE_TYPES, allEntries = true)
  public void evictCachedDocumentFileTypes() {
    log.info("DocumentFileTypes removed from cache");
  }

  @Scheduled(cron = "0 0 */12 * * *")
  @CacheEvict(AouCacheNames.CN_INDIVIDU)
  public void clearCnIndividuCache() {
    log.info("Cache '{}' cleared.", AouCacheNames.CN_INDIVIDU);
  }

}