package ch.unige.aou.service.rest.abstractservice;

import java.util.List;
import java.util.Map;

import org.springframework.web.util.UriComponentsBuilder;

import ch.unige.solidify.service.RemoteResourceService;
import ch.unige.solidify.service.SolidifyRestClientService;

import ch.unige.aou.config.AouProperties;
import ch.unige.aou.model.publication.DocumentFile;
import ch.unige.aou.rest.AouActionName;
import ch.unige.aou.rest.ResourceName;

public abstract class DocumentFileRemoteResourceService extends RemoteResourceService<DocumentFile> {

  final protected String adminModuleUrl;

  protected DocumentFileRemoteResourceService(SolidifyRestClientService restClientService, AouProperties aouProperties) {
    super(restClientService);

    this.adminModuleUrl = aouProperties.getModule().getAdmin().getUrl();
  }

  public String fulltext(String resId) {
    String url = this.getResourceUrl() + "/" + resId + "/" + AouActionName.FULLTEXT;
    return this.restClientService.getResource(url, String.class);
  }

  public List<DocumentFile> listByPublication(String publicationId, Map<String, String> queryParameters) {
    String url = this.adminModuleUrl + "/" + ResourceName.PUBLICATIONS + "/" + publicationId + "/" + ResourceName.DOCUMENT_FILES;
    UriComponentsBuilder uriBuilder = UriComponentsBuilder.fromUriString(url);
    for (Map.Entry<String, String> queryParam : queryParameters.entrySet()) {
      uriBuilder.queryParam(queryParam.getKey(), queryParam.getValue());
    }
    url = uriBuilder.toUriString();
    return this.restClientService.getAllResources(url, DocumentFile.class);
  }

  @Override
  protected Class<DocumentFile> getResourceClass() {
    return DocumentFile.class;
  }

  @Override
  protected String getResourceUrl() {
    return this.adminModuleUrl + "/" + ResourceName.DOCUMENT_FILES;
  }
}
