package ch.unige.aou.service.rest.abstractservice;

import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.domain.Pageable;

import ch.unige.solidify.rest.RestCollection;
import ch.unige.solidify.service.SolidifyRestClientService;

import ch.unige.aou.config.AouProperties;
import ch.unige.aou.model.publication.PublicationSubtype;
import ch.unige.aou.model.publication.PublicationSubtypeDTO;
import ch.unige.aou.model.publication.PublicationSubtypeDocumentFileTypeDTO;
import ch.unige.aou.rest.AouActionName;
import ch.unige.aou.rest.ResourceName;
import ch.unige.aou.service.AouCacheNames;

public abstract class PublicationSubtypeRemoteResourceService extends AdminRemoteResourceService<PublicationSubtype> {

  protected PublicationSubtypeRemoteResourceService(SolidifyRestClientService restClientService, AouProperties aouProperties) {
    super(restClientService, aouProperties);
  }

  /**
   * Getting a list of PublicationSubtypeDTO instead of PublicationSubtype class allows to correctly retrieve the language code
   *
   * @param pageable
   * @return
   */
  @Cacheable(AouCacheNames.PUBLICATION_SUBTYPES)
  public RestCollection<PublicationSubtypeDTO> findAllDTO(Pageable pageable) {
    return this.restClientService.getResourceList(this.getResourceUrl(), PublicationSubtypeDTO.class, pageable);
  }

  @Cacheable(AouCacheNames.PUBLICATION_SUBTYPES)
  public RestCollection<PublicationSubtypeDocumentFileTypeDTO> findAllPublicationSubtypeDocumentFileTypesDTO(String publicationSubtypeId,
          Pageable pageable) {
    String url = this.getResourceUrl() + "/" + publicationSubtypeId + "/" + AouActionName.LIST_DOCUMENT_FILE_TYPES;
    return this.restClientService.getResourceList(url, PublicationSubtypeDocumentFileTypeDTO.class, pageable);
  }

  @Override
  protected Class<PublicationSubtype> getResourceClass() {
    return PublicationSubtype.class;
  }

  @Override
  protected String getResourceUrl() {
    return this.adminModuleUrl + "/" + ResourceName.PUBLICATION_SUBTYPES;
  }
}
