package ch.unige.aou.service.rest.abstractservice;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.util.UriComponentsBuilder;

import ch.unige.solidify.exception.SolidifyRuntimeException;
import ch.unige.solidify.rest.RestCollection;
import ch.unige.solidify.rest.RestCollectionPage;
import ch.unige.solidify.service.SolidifyRestClientService;

import ch.unige.aou.config.AouProperties;
import ch.unige.aou.model.security.User;
import ch.unige.aou.model.security.UserDisplayableInfos;
import ch.unige.aou.rest.AouActionName;
import ch.unige.aou.rest.ResourceName;
import ch.unige.aou.service.AouCacheNames;

public abstract class UserRemoteResourceService extends AdminRemoteResourceService<User> {

  private static final Logger log = LoggerFactory.getLogger(UserRemoteResourceService.class);

  public UserRemoteResourceService(SolidifyRestClientService restClientService, AouProperties aouProperties) {
    super(restClientService, aouProperties);
  }

  public User getAuthenticatedUser() {
    String url = this.adminModuleUrl + "/" + ResourceName.USERS + "/" + AouActionName.AUTHENTICATED;
    return this.restClientService.getResource(url, User.class);
  }

  public UserDisplayableInfos getUserDisplayableInfos(String externalUid) {
    String url = UriComponentsBuilder.fromUriString(
            this.adminModuleUrl + "/" + ResourceName.USERS + "/" + AouActionName.GET_USER_DISPLAYABLE_INFOS + "/" + externalUid).toUriString();
    return this.restClientService.getResource(url, UserDisplayableInfos.class);
  }

  @Cacheable(AouCacheNames.USER_EXTERNAL_UID)
  public User findByExternalUid(String externalUid) {
    try {
      final List<User> users = this.getUsersByExternalUid(externalUid);

      if (users.size() == 1) {
        return users.get(0);
      } else if (users.size() > 1) {
        throw new SolidifyRuntimeException("more than one user correspond to the externalUid");
      }

    } catch (final Exception e) {
      log.error("Error finding external UID " + externalUid, e);
    }

    return null;
  }

  public List<User> getUsersByExternalUid(String externalUid) {
    final Pageable page = PageRequest.of(0, RestCollectionPage.DEFAULT_SIZE_PAGE);
    // Build URL
    String url = this.adminModuleUrl + "/" + ResourceName.USERS;
    final UriComponentsBuilder uriBuilder = UriComponentsBuilder.fromUriString(url);
    final MultiValueMap<String, String> params = new LinkedMultiValueMap<>();
    params.set("externalUid", externalUid);
    uriBuilder.queryParams(params);
    url = uriBuilder.toUriString();
    // Get User RestCollection
    RestCollection<User> userCollection = this.restClientService.getResourceList(url, User.class, page);
    // Return user list
    return userCollection.getData();
  }

  @Override
  protected Class<User> getResourceClass() {
    return User.class;
  }

  @Override
  protected String getResourceUrl() {
    return this.adminModuleUrl + "/" + ResourceName.USERS;
  }
}
