package ch.unige.aou.service.security;

import java.util.Collection;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;

import ch.unige.solidify.auth.model.ApplicationRole;
import ch.unige.solidify.auth.model.AuthApplicationRole;
import ch.unige.solidify.exception.SolidifyRuntimeException;

import ch.unige.aou.controller.AouControllerAction;

public abstract class AbstractPermissionService {

  private static final Logger logger = LoggerFactory.getLogger(AbstractPermissionService.class);
  
  protected boolean canBypassPermissionCheck() {
    final String[] authorizedApplicationRoleNames = { AuthApplicationRole.ADMIN_ID, AuthApplicationRole.ROOT_ID,
            AuthApplicationRole.TRUSTED_CLIENT_ID };

    final Collection<? extends GrantedAuthority> authorities = this.getAuthentication().getAuthorities();

    for (final String roleName : authorizedApplicationRoleNames) {
      for (final GrantedAuthority authority : authorities) {
        if (authority.toString().equals(roleName)) {
          return true;
        }
      }
    }
    return false;
  }

  protected boolean hasUserApplicationRole() {
    return this.hasApplicationRole(AuthApplicationRole.USER);
  }

  protected boolean hasApplicationRole(ApplicationRole applicationRole) {
    final Collection<? extends GrantedAuthority> authorities = this.getAuthentication().getAuthorities();
    return authorities.stream().anyMatch(a -> a.toString().equals(applicationRole.getResId()));
  }

  protected Authentication getAuthentication() {
    return SecurityContextHolder.getContext().getAuthentication();
  }

  protected AouControllerAction getControllerAction(Object permission) {
    try {
      return AouControllerAction.valueOf(permission.toString());
    } catch (final Exception ex) {
      logger.error("Unable to check for access permission: the action name '{}' is wrong", permission);
      throw new SolidifyRuntimeException("unable to check for access permission: the action name '" + permission + "' is wrong");
    }
  }
}
