package ch.unige.aou.test.doc;

import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.context.SpringBootTest.WebEnvironment;
import org.springframework.context.annotation.Import;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import ch.unige.solidify.test.doc.RestDocsDocumentation;

import ch.unige.aou.rest.UrlPath;
import ch.unige.aou.test.resource.ApplicationRoleDoc;
import ch.unige.aou.test.resource.ContributorDoc;
import ch.unige.aou.test.resource.DocumentFileTypeDoc;
import ch.unige.aou.test.resource.EventDoc;
import ch.unige.aou.test.resource.GlobalBannerDoc;
import ch.unige.aou.test.resource.InstitutionDoc;
import ch.unige.aou.test.resource.LanguageDoc;
import ch.unige.aou.test.resource.LicenseDoc;
import ch.unige.aou.test.resource.LicenseGroupDoc;
import ch.unige.aou.test.resource.ModuleDoc;
import ch.unige.aou.test.resource.NotificationDoc;
import ch.unige.aou.test.resource.OaiSetDoc;
import ch.unige.aou.test.resource.PersonDoc;
import ch.unige.aou.test.resource.PublicationDoc;
import ch.unige.aou.test.resource.PublicationTypeDoc;
import ch.unige.aou.test.resource.ResearchGroupDoc;
import ch.unige.aou.test.resource.RoleDoc;
import ch.unige.aou.test.resource.StructureDoc;
import ch.unige.aou.test.resource.SystemPropertyDoc;
import ch.unige.aou.test.resource.UserDoc;

@ActiveProfiles({ "test", "sec-noauth", "person-search-aou-prod", "client-standalone", "storage-file", "index-inmemory" })
@ExtendWith(SpringExtension.class)
@Import(SolutionRestTestsAndDocumentationConfig.class)
@SpringBootTest(webEnvironment = WebEnvironment.DEFINED_PORT, properties = { "spring.cloud.bootstrap.enabled=true" })
public class SolutionRestTestsAndDocumentation extends RestDocsDocumentation {

  @Override
  protected void loadResources() {
    this.addResources(new InstitutionDoc());
    this.addResources(new LanguageDoc());
    this.addResources(new LicenseDoc());
    this.addResources(new LicenseGroupDoc());
    this.addResources(new ModuleDoc());
    this.addResources(new OaiSetDoc());
    this.addResources(new PersonDoc());
    this.addResources(new PublicationDoc());
    this.addResources(new PublicationTypeDoc());
    this.addResources(new ApplicationRoleDoc());
    this.addResources(new SystemPropertyDoc());
    this.addResources(new UserDoc());
    this.addResources(new ResearchGroupDoc());
    this.addResources(new RoleDoc());
    this.addResources(new StructureDoc());
    this.addResources(new EventDoc());
    this.addResources(new NotificationDoc());
    this.addResources(new ContributorDoc());
    this.addResources(new DocumentFileTypeDoc());
    this.addResources(new GlobalBannerDoc());
  }

  @Override
  protected void setUrlIgnoreList() {
    // Admin
    this.urlIgnoreList.add(this.getContextPath() + UrlPath.ADMIN_MONITOR);

    // Access
    this.urlIgnoreList.add(this.getContextPath() + UrlPath.ACCESS_OAI_PROVIDER);
  }

  @Override
  protected void setUrlReadOnlyList() {
    // Admin
    this.urlReadOnlyList.add(this.getContextPath() + UrlPath.ADMIN_SYSTEM_PROPERTIES);
    this.urlReadOnlyList.add(this.getContextPath() + UrlPath.ADMIN_MODULES);
  }
}
