package ch.unige.aou.test.resource;

import static org.springframework.restdocs.payload.PayloadDocumentation.subsectionWithPath;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.restdocs.payload.FieldDescriptor;
import org.springframework.restdocs.snippet.Attributes;

import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.unige.aou.model.notification.EventType;
import ch.unige.aou.rest.UrlPath;

public class EventDoc extends AbstractResourceDoc {
  @Override
  public List<String> getListLinks() {
    return Collections.emptyList();
  }

  @Override
  public String getName() {
    return UrlPath.ADMIN_EVENTS;
  }

  @Override
  public List<String> getResourceLinks() {
    return Collections.emptyList();
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    // @formatter:off
    final List<FieldDescriptor> documentationFields = new ArrayList<>(Arrays.asList(
            new FieldDescriptorBuilder(this)
                    .name("message")
                    .description("The message of the event")
                    .mandatory(true)
                    .type(TypeField.STRING)
                    .build(),
            subsectionWithPath("triggerBy")
                    .description("The person who triggers of the event")
                    .attributes(new Attributes.Attribute(READ_ONLY_VALUE, String.valueOf(true))),
            new FieldDescriptorBuilder(this)
                    .name("eventType.resId")
                    .description("The type of the event")
                    .mandatory(true)
                    .type(TypeField.STRING)
                    .enumValues(EventType.getAllEventTypes().stream().map(t->t.toString()).collect(Collectors.toList()))
                    .customConstraints(CONSTRAINT_VALID_JSON)
                    .testValue("PUBLICATION_CREATED")
                    .readOnly(true)
                    .build(),
            subsectionWithPath("eventType").description("The type of the event")
                    .attributes(new Attributes.Attribute(READ_ONLY_VALUE, String.valueOf(true)),
                            new Attributes.Attribute(MANDATORY_VALUE, String.valueOf(true))),
            new FieldDescriptorBuilder(this)
                    .name("publication.resId")
                    .description("The publication linked to the event")
                    .mandatory(true)
                    .type(TypeField.STRING)
                    .testValue(TEST_STRING_VALUE)
                    .readOnly(true)
                    .build(),
            subsectionWithPath("publication")
                    .description("The publication of event")
                    .attributes(new Attributes.Attribute(READ_ONLY_VALUE, String.valueOf(true)),
                            new Attributes.Attribute(MANDATORY_VALUE, String.valueOf(true)))
    ));

    // @formatter:on
    return documentationFields;
  }

}
